#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <unistd.h>
#include <errno.h>
#include "cmd.h"
#include "proc.h"
#include "buffer.h"


static void (*cmd_out)(void *,char *,int);
static void (*cmd_err)(void *,char *,int);
static void *cmd_out_arg;

static buffer_t *buf_e, *buf_o;

static int nargs,sargs;
static char **args;

static int ispc(int x) { return (x>0 && x<=32); }

static void put_arg(char *data, int dup)
{
  if (dup) data= strdup(data);
  if (nargs==sargs)
  {
    sargs += sargs/2 + 10;
    args= realloc(args, sizeof(args[0])*sargs);
  }
  args[nargs]= data;
  nargs++;
}

void cmd_reset()
{
  int i;
  for(i=0;i<nargs;i++) free(args[i]);
  nargs= 0;
  buf_o= buffer_new(512);
  buf_e= buffer_new(512);
}

void cmd_args(char *data,...)
{
  va_list ap;
  char *d;
  put_arg(data,1);
  va_start(ap,data);
  while(1)
  {
    d= va_arg(ap,char*);
    if (!d) break;
    put_arg(d,1);
  }
  va_end(ap);
}

void cmd_parse(char *data)
{
  char *e,*n,*d;
  int L;
  if (!data) return ;
again:
  while(ispc(*data)) data++;
  if (!*data) return ;
  if (data[0]=='\'')
  {
    data++;
    for(e=data;e[0];e++) if (e[0]=='\'') break;
    n= e+1;
  }
  else
  {
    for(e=data;e[0];e++) if (ispc(e[0])) break;
    n= e;
  }
  L= e-data;
  d= malloc(L+1);
  memcpy(d,data,L);
  d[L]= 0;
  data= n;
  put_arg(d,0);
  goto again;
}


void cmd_set_out( void (*out)(void *,char *,int),
                  void (*err)(void *,char *,int),
                  void *out_arg)
{
  cmd_out= out;
  cmd_err= err;
  cmd_out_arg= out_arg;
}

static int feed_data
  (buffer_t **bb, void (*f)(void *,char*,int), char *data,int len)
{
  int i;
  int have_nl;
  char *o;
  size_t L;

  have_nl= 0;
  for(i=0;i<len;i++)  if (data[i]=='\n') { have_nl=1; break; }
  if (have_nl) i++;
  buffer_write(*bb, data, i);

  if (have_nl)
  {
    buffer_write(*bb, "", 1);
    buffer_free(*bb, (void**) &o, &L);
    *bb= buffer_new(512);
    if (f) (*f)(cmd_out_arg, o, L-1);
    free(o);
  }
  return i;
}

static void read_fd(int fd, buffer_t **bb, void (*f)(void *,char*,int))
{
  char data[512]; int R,k;
  char *d;
  R= read(fd, data, sizeof(data));
  if (R<=0) return ;
  d= data;
  while(R)
  {
    k= feed_data(bb, f, d, R);
    R-= k;
    d+= k;
  }
}

int cmd_run()
{
  fd_set rset, xset;
  int code;
  int id;
  int fd_o, fd_e, fd_max;
  if (nargs==0) 
  {
    return 1; 
  }
  put_arg(NULL, 0);
  id= proc_start(args[0], args, PROC_OUT | PROC_ERR);
  if (id<0)
  {
    return 1;
  }

  fd_o= proc_output(id);
  fd_e= proc_error(id);
  fd_max= fd_o > fd_e ? fd_o: fd_e;
  
  while(proc_islive(id))
  {
    FD_ZERO(&rset); FD_ZERO(&xset);
    FD_SET(fd_o, &rset);  FD_SET(fd_o, &xset);
    FD_SET(fd_e, &rset);  FD_SET(fd_e, &xset);
    if (select(fd_max+1, &rset, NULL, &xset, NULL)<0)
    {
      if (errno==EINTR) continue;
      break;
    }
    if (FD_ISSET(fd_o, &xset) || FD_ISSET(fd_e, &xset)) break;
    if (FD_ISSET(fd_o, &rset)) read_fd(fd_o, &buf_o, cmd_out);
    if (FD_ISSET(fd_e, &rset)) read_fd(fd_e, &buf_e, cmd_err);
  }
  
  return proc_end(id, &code);
}

#ifdef cmd_self_test

void oo(void *A, char *K, int L)
{
  printf("O>%s",K);
}

void ee(void *A,char *K, int L)
{
  printf("E>%s",K);
}

int main(int argc,char **argv)
{
  int i;
  cmd_set_out(oo,ee,NULL);
  cmd_reset();
  for(i=1;argv[i];i++) cmd_args(argv[i],NULL);
  printf("R=%d\n",cmd_run());
  return 0;
}
#endif
