#include <stdint.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include "gfx.h"
#include "bfile.h"
#include "path.h"
#include "main.h"

typedef struct
{ 
  int widget;
  int offset;
} str_option_t;

#define OFFSET(x)  ((int)(size_t)(&(((bfile_t*)0)->x)))

static str_option_t str_options[]=
{
  { WopaTOOLDIR,                OFFSET(a_build_tool_dir) },
  { WopaTOOLaapt,               OFFSET(a_aapt) },
  { WopaTOOLdx,                 OFFSET(a_dx) },
  { WopaTOOLzipalign,           OFFSET(a_zipalign) },
  { WopaJAR,                    OFFSET(a_jarfile) },
  { WopaJAVAC,                  OFFSET(a_javac) },
  { WopaKEYTOOL,                OFFSET(a_keytool) },
  { WopaJARSIGNER,              OFFSET(a_jarsigner) },
  { WopaADB,                    OFFSET(a_adb) },

  { WopcCOMPILER,               OFFSET(c_compiler_path) },
  { WopcCOMPILERoptions,        OFFSET(c_compiler_options) },
  { WopcCOMPILERoptionsR,       OFFSET(c_compiler_optionsR) },
  { WopcCOMPILERoptionsD,       OFFSET(c_compiler_optionsD) },
  { WopcLINKERoptions,          OFFSET(c_linker_options) },
  { WopcLINKERoptionsR,         OFFSET(c_linker_optionsR) },
  { WopcLINKERoptionsD,         OFFSET(c_linker_optionsD) },

  { WoppPACKAGE,                OFFSET(a_package) },
  { WoppAPPNAME,                OFFSET(a_appname) },
  { -1,                         -1 },
};

static char* perms[]=
{
  "ACCEPT_HANDOVER",
  "ACCESS_BACKGROUND_LOCATION",
  "ACCESS_CHECKIN_PROPERTIES",
  "ACCESS_COARSE_LOCATION",
  "ACCESS_FINE_LOCATION",
  "ACCESS_LOCATION_EXTRA_COMMANDS",
  "ACCESS_MEDIA_LOCATION",
  "ACCESS_NETWORK_STATE",
  "ACCESS_NOTIFICATION_POLICY",
  "ACCESS_WIFI_STATE",
  "ACCOUNT_MANAGER",
  "ACTIVITY_RECOGNITION",
  "ADD_VOICEMAIL",
  "ANSWER_PHONE_CALLS",
  "BATTERY_STATS",
  "BIND_ACCESSIBILITY_SERVICE",
  "BIND_APPWIDGET",
  "BIND_AUTOFILL_SERVICE",
  "BIND_CALL_REDIRECTION_SERVICE",
  "BIND_CARRIER_MESSAGING_CLIENT_SERVICE",
  "BIND_CARRIER_MESSAGING_SERVICE",
  "BIND_CARRIER_SERVICES",
  "BIND_CHOOSER_TARGET_SERVICE",
  "BIND_CONDITION_PROVIDER_SERVICE",
  "BIND_DEVICE_ADMIN",
  "BIND_DREAM_SERVICE",
  "BIND_INCALL_SERVICE",
  "BIND_INPUT_METHOD",
  "BIND_MIDI_DEVICE_SERVICE",
  "BIND_NFC_SERVICE",
  "BIND_NOTIFICATION_LISTENER_SERVICE",
  "BIND_PRINT_SERVICE",
  "BIND_QUICK_SETTINGS_TILE",
  "BIND_REMOTEVIEWS",
  "BIND_SCREENING_SERVICE",
  "BIND_TELECOM_CONNECTION_SERVICE",
  "BIND_TEXT_SERVICE",
  "BIND_TV_INPUT",
  "BIND_VISUAL_VOICEMAIL_SERVICE",
  "BIND_VOICE_INTERACTION",
  "BIND_VPN_SERVICE",
  "BIND_VR_LISTENER_SERVICE",
  "BIND_WALLPAPER",
  "BLUETOOTH",
  "BLUETOOTH_ADMIN",
  "BLUETOOTH_PRIVILEGED",
  "BODY_SENSORS",
  "BROADCAST_PACKAGE_REMOVED",
  "BROADCAST_SMS",
  "BROADCAST_STICKY",
  "BROADCAST_WAP_PUSH",
  "CALL_COMPANION_APP",
  "CALL_PHONE",
  "CALL_PRIVILEGED",
  "CAMERA",
  "CAPTURE_AUDIO_OUTPUT",
  "CHANGE_COMPONENT_ENABLED_STATE",
  "CHANGE_CONFIGURATION",
  "CHANGE_NETWORK_STATE",
  "CHANGE_WIFI_MULTICAST_STATE",
  "CHANGE_WIFI_STATE",
  "CLEAR_APP_CACHE",
  "CONTROL_LOCATION_UPDATES",
  "DELETE_CACHE_FILES",
  "DELETE_PACKAGES",
  "DIAGNOSTIC",
  "DISABLE_KEYGUARD",
  "DUMP",
  "EXPAND_STATUS_BAR",
  "FACTORY_TEST",
  "FOREGROUND_SERVICE",
  "GET_ACCOUNTS",
  "GET_ACCOUNTS_PRIVILEGED",
  "GET_PACKAGE_SIZE",
  "GET_TASKS",
  "GLOBAL_SEARCH",
  "INSTALL_LOCATION_PROVIDER",
  "INSTALL_PACKAGES",
  "INSTALL_SHORTCUT",
  "INSTANT_APP_FOREGROUND_SERVICE",
  "INTERNET",
  "KILL_BACKGROUND_PROCESSES",
  "LOCATION_HARDWARE",
  "MANAGE_DOCUMENTS",
  "MANAGE_OWN_CALLS",
  "MASTER_CLEAR",
  "MEDIA_CONTENT_CONTROL",
  "MODIFY_AUDIO_SETTINGS",
  "MODIFY_PHONE_STATE",
  "MOUNT_FORMAT_FILESYSTEMS",
  "MOUNT_UNMOUNT_FILESYSTEMS",
  "NFC",
  "NFC_TRANSACTION_EVENT",
  "PACKAGE_USAGE_STATS",
  "PERSISTENT_ACTIVITY",
  "PROCESS_OUTGOING_CALLS",
  "READ_CALENDAR",
  "READ_CALL_LOG",
  "READ_CONTACTS",
  "READ_EXTERNAL_STORAGE",
  "READ_INPUT_STATE",
  "READ_LOGS",
  "READ_PHONE_NUMBERS",
  "READ_PHONE_STATE",
  "READ_SMS",
  "READ_SYNC_SETTINGS",
  "READ_SYNC_STATS",
  "READ_VOICEMAIL",
  "REBOOT",
  "RECEIVE_BOOT_COMPLETED",
  "RECEIVE_MMS",
  "RECEIVE_SMS",
  "RECEIVE_WAP_PUSH",
  "RECORD_AUDIO",
  "REORDER_TASKS",
  "REQUEST_COMPANION_RUN_IN_BACKGROUND",
  "REQUEST_COMPANION_USE_DATA_IN_BACKGROUND",
  "REQUEST_DELETE_PACKAGES",
  "REQUEST_IGNORE_BATTERY_OPTIMIZATIONS",
  "REQUEST_INSTALL_PACKAGES",
  "REQUEST_PASSWORD_COMPLEXITY",
  "RESTART_PACKAGES",
  "SEND_RESPOND_VIA_MESSAGE",
  "SEND_SMS",
  "SET_ALARM",
  "SET_ALWAYS_FINISH",
  "SET_ANIMATION_SCALE",
  "SET_DEBUG_APP",
  "SET_PREFERRED_APPLICATIONS",
  "SET_PROCESS_LIMIT",
  "SET_TIME",
  "SET_TIME_ZONE",
  "SET_WALLPAPER",
  "SET_WALLPAPER_HINTS",
  "SIGNAL_PERSISTENT_PROCESSES",
  "SMS_FINANCIAL_TRANSACTIONS",
  "START_VIEW_PERMISSION_USAGE",
  "STATUS_BAR",
  "SYSTEM_ALERT_WINDOW",
  "TRANSMIT_IR",
  "UNINSTALL_SHORTCUT",
  "UPDATE_DEVICE_STATS",
  "USE_BIOMETRIC",
  "USE_FINGERPRINT",
  "USE_FULL_SCREEN_INTENT",
  "USE_SIP",
  "VIBRATE",
  "WAKE_LOCK",
  "WRITE_APN_SETTINGS",
  "WRITE_CALL_LOG",
  "WRITE_CONTACTS",
  "WRITE_EXTERNAL_STORAGE",
  "WRITE_GSERVICES",
  "WRITE_SECURE_SETTINGS",
  "WRITE_SETTINGS",
  "WRITE_SYNC_SETTINGS",
  "WRITE_VOICEMAIL",
  NULL,
};

static bfile_t *BF;
static c_source_t *cs_selected;
static c_dep_t *cd_selected;
static strlist_t *sel_c_library, *sel_c_include;
static int sel_c_library_idx, sel_c_include_idx;

static char *gethomedir()
{
  char *Z;
  Z= getenv("HOME");
  return Z ? Z: "/";
}

static void show_string_values(bfile_t *bf)
{
  int i;
  char **K;
  for(i=0;str_options[i].widget>0;i++)
  {
     K= (char**) ((char*) bf + str_options[i].offset);
     if (!K[0]) continue;
     gfx_set_text(str_options[i].widget, K[0]);
  }
}


static void get_string_values(bfile_t *bf)
{
  char *V;
  int i;
  char **K;
  for(i=0;str_options[i].widget>0;i++)
  {
     V= gfx_get_text(str_options[i].widget);
     if (isempty(V)) continue;
     K= (char**) ((char*) bf + str_options[i].offset);
     K[0]= strdup(V);
  }
}

static void show_c_sources(bfile_t *bf)
{
  c_source_t *cs;
  gfx_filelist_set_directory(WopdBROWSE, bf->dir);
  for(cs=bf->c_sources.next;cs!=&bf->c_sources;cs=cs->next)
    gfx_insert(WopdSOURCES,-1, cs->name, cs, NULL);
  gfx_clear(WopdDEPENDENCIES);
  cs_selected= NULL;
  cd_selected= NULL;
}

static void show_permissions(bfile_t *bf)
{
  int i,V,R;
  strlist_t *P;
//
// FIXME: we have a bug. when the first item inserted in a list
// has an icon, the icon doesn't get displayed..
// baah.
//
  gfx_insert(WoppPERMS, -1, "INTERNET", NULL, res0);
  gfx_remove(WoppPERMS, 0, NULL);
  P= bf->perms.next;
  for(i=0;perms[i];i++)
  {
    if (P==&bf->perms) V= 0;
    else
    {
      R= strcmp(P->name,perms[i]);
      if (R==0) V=1; 
      if (R<0) { i--; P= P->next; continue; }
      V= 0;
    }
    gfx_insert(WoppPERMS, -1, perms[i], perms[i], V?check16:remove16);
  }
}

static void show_dirs(bfile_t *bf, int widget, strlist_t *L)
{
  strlist_t *K;
  for(K=L->next;K!=L;K=K->next)
    gfx_insert(widget, -1, K->name, K, NULL);
}

static void show_values(bfile_t *bf)
{
  BF= bf;

  show_string_values(bf);
  show_c_sources(bf);
  show_permissions(bf);
  show_dirs(bf, WopcINCLUDES, &bf->c_include_paths);
  show_dirs(bf, WopcLIBRARIES, &bf->c_include_paths);
  sel_c_library= NULL;
  sel_c_include= NULL;
  if (bf->scalign=='P') gfx_set_state(WoppPORTRAIT,1);
                   else gfx_set_state(WoppLANDSCAPE,1);
}

static void select_c_source(c_source_t *cs)
{
  c_dep_t *cd;
  if (cs_selected==cs) return ;
  cs_selected= cs;
  gfx_clear(WopdDEPENDENCIES);
  for(cd=cs->dhead.next;cd!=&cs->dhead;cd=cd->next)
    gfx_insert(WopdDEPENDENCIES, -1, cd->name, cd, NULL);
  cd_selected= NULL;
}

static void select_c_dependency(c_dep_t *cd)
{
  cd_selected= cd;
}

static void delete_c_dependency()
{
  int i;
  c_dep_t *cd;
  if (!cd_selected || !cs_selected) return ;
  for(i=0, cd=cs_selected->dhead.next;cd!=&cs_selected->dhead;cd=cd->next,i++)
    if (cd==cd_selected) goto found;
  return ;
found:
  gfx_remove(WopdDEPENDENCIES,i,NULL);
  cd->prev->next= cd->next;
  cd->next->prev= cd->prev;
  free(cd->name);
  free(cd);
  cd_selected= NULL;
}

static int opscreen=-1;

static void set_op_screen(int sno)
{
  if (sno==opscreen) return ;
  gfx_set_selected(WopLIST, sno);
  gfx_set_selected(WopPANEL, sno);
  opscreen= sno;
}

static int toggle_perm(bfile_t *bf,char *name)
{
  int R;
  R= bfile_has_perm(bf,name);
  if (R) bfile_remove_perm(bf,name);
    else bfile_add_perm(bf,name);
  return R^1;
}


static void ss_setdir(char *s)
{
  gfx_dirlist_set_directory(WdsLIST, s);
  gfx_set_text(WdsDIR, s);
}

char *select_dir(char *init,char *msg)
{
  int R;
  char *rv;
  if (!init) init= gethomedir();
  R= 0;
  gfx_SCR_seldir();
  rv= NULL;
  ss_setdir(init);
  gfx_set_text(WdsMSG, msg);
  while(!R)
  {
    gfx_event_t event;
    if (!gfx_event(NULL,&event)) continue;
    switch(event.type)
    {
    case GFXEV_QUIT:
    case WdsCANCEL: R= 2; break;
    case WdsOK:     R= 1; break;
    case WdsHOME:   ss_setdir(gethomedir()); break;
    case WdsTOP:    ss_setdir("/"); break;
    case WdsLIST:   gfx_set_text(WdsDIR, event.string); break;
    }
  }
  if (R==1) rv= gfx_dirlist_get_directory(WdsLIST);
       else rv= NULL;
  gfx_leave_screen();
  return rv;
}

static void browse_tool(int widget)
{
  char *fn;
  fn= gfx_get_text(widget);
  fn= gfx_select_file(0, fn, NULL);
  if (!fn) return ;
  gfx_set_text(widget, fn);
}

static void browse_dir(int widget, char *msg)
{
  char *fn;
  fn= gfx_get_text(widget);
  fn= select_dir(fn, msg);
  if (!fn) return ;
  gfx_set_text(widget, fn);
}


void options_screen(bfile_t *bf)
{
  strlist_t *SN;
  gfx_event_t event;
  c_dep_t *cd;
  int i,V;
  int q=0;
  char *fn;

  gfx_SCR_options();
  char *sets[]= { "C Compiler",
                  "C Linker",
                  "C Sources", 
                  "NDK & SDK Env",
                  "Project Settings",
                  "Permissions",
                  NULL };
  for(i=0;sets[i];i++) gfx_insert(WopLIST, -1, sets[i], NULL, NULL);
  set_op_screen(0);
  gfx_filelist_set_directory(WopdBROWSE, bf->dir);
  gfx_set_text(WopFILE, bf->dispname);
  show_values(bf);

  while(!q) if (gfx_event(NULL,&event)) switch(event.type)
  {
  case GFXEV_QUIT:
  case WopOK: q= 1; break;
  case WopLIST: set_op_screen(event.key); break;
  case WopcSETcompiler:
      fn= gfx_select_file(0,bf->fn, NULL);
      if (!fn) break;
      gfx_set_text(WopcCOMPILER,fn);
      bf->c_compiler_path= fn;
      break;
  case WoppPERMS:
      V= toggle_perm(bf,event.usr);
      gfx_remove(WoppPERMS,event.key,NULL);
      gfx_insert(WoppPERMS,event.key, event.usr, event.usr, V?check16:remove16);
      break;
  case WopdSOURCEadd:
      fn= gfx_filelist_get_selected(WopdBROWSE);
      if (!fn) break;
      fn= path_rel(fn,bf->dir);
      gfx_insert(WopdSOURCES, -1, fn, bfile_add_c_source(bf, fn, -1), NULL);
      break;
  case WopdSOURCES:
      select_c_source(event.usr);
      break;
  case WopdDEPadd:
      if (!cs_selected) break;
      fn= gfx_filelist_get_selected(WopdBROWSE);
      if (!fn) break;
      fn= path_rel(fn,bf->dir);
      cd= bfile_add_c_dependency(bf, cs_selected, fn);
      gfx_insert(WopdDEPENDENCIES, -1, cd->name,cd,  NULL);
      break;
  case WopdDEPremove:
      delete_c_dependency();
      break;
  case WopdDEPENDENCIES:
      select_c_dependency(event.usr);
      break;
  case WopcLIBRARIES:
      sel_c_library= event.usr;
      sel_c_library_idx= event.key;
      break;
  case WopcINCLUDES:
      sel_c_include= event.usr;
      sel_c_include_idx= event.key;
      break;
  case WopcINCLUDESadd:
      fn= select_dir(bf->dir, "Select Include Directory (Relative)");
      if (!fn) continue;
      fn= path_rel(fn, bf->dir);
      SN= bfile_add_c_include(bf, fn);
      gfx_insert(WopcINCLUDES, -1, SN->name, SN, NULL);
      break;
  case WopcINCLUDESabs:
      fn= select_dir(bf->dir, "Select Include Directory (Absolute)");
      if (!fn) continue;
      SN= bfile_add_c_include(bf, fn);
      gfx_insert(WopcINCLUDES, -1, SN->name, SN, NULL);
      break;
  case WopcINCLUDESremove:
      if (!sel_c_include) break;
      gfx_remove(WopcINCLUDES, sel_c_include_idx, NULL);
      strnode_remove(sel_c_include);
      sel_c_include= NULL;
      break;
  case WopcLIBRARIESremove:
      if (!sel_c_library) break;
      gfx_remove(WopcLIBRARIES, sel_c_library_idx, NULL);
      strnode_remove(sel_c_library);
      sel_c_library= NULL;
      break;
  case WopcLIBRARIESadd:
      fn= select_dir(bf->dir, "Select Library Directory");
      if (!fn) continue;
      // fn= path_rel(fn, bf->dir);
      SN= bfile_add_c_library(bf, fn);
      gfx_insert(WopcLIBRARIES, -1, SN->name, SN, NULL);
      break;
  case WoppLANDSCAPE: bf->scalign= 'L'; break;
  case WoppPORTRAIT:  bf->scalign= 'P'; break;

  case WopaTOOLDIRb: browse_dir(WopaTOOLDIR, "Build tool directory"); break;
  case WopaTOOLaaptB: browse_tool(WopaTOOLaapt); break;
  case WopaTOOLdxB: browse_tool(WopaTOOLdx); break;
  case WopaTOOLzipalignB: browse_tool(WopaTOOLzipalign); break;
  case WopaJARb: browse_tool(WopaJAR); break;
  case WopaJAVACb: browse_tool(WopaJAVAC); break;
  case WopaKEYTOOLb: browse_tool(WopaKEYTOOL); break;
  case WopaJARSIGNERb: browse_tool(WopaJARSIGNER); break;
  case WopaADBb: browse_tool(WopaADB); break;
  }
  get_string_values(bf);
  gfx_leave_screen();
}


