#include <stdio.h>
#include <unistd.h>
#include <stdint.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include "gfx.h"
#include "path.h"
#include "bfile.h"
#include "main.h"
#include "cmd.h"


#define OFFSET(x)  ((int)(size_t)(&(((bfile_t*)0)->x)))


static void (*cmd_logf)(char *,int );

int isempty(char *K)
{
  int i;
  if (K) for(i=0;K[i];i++) if (K[i]>' ') return 0;
  return 1;
}

static bfile_t *load_file(bfile_t *bf)
{
  char *fn;
  static char *types[]= { "Build Files (*.abf)", NULL };
  bfile_save(bf);
  fn= gfx_select_file(0,bf->fn, types);
  if (!fn) return bf;
  bfile_destroy(bf);
  bf= bfile_open(fn);
  gfx_set_text(Wfile, bf->dispname);
  return bf;
}

static int check_if_template(bfile_t *bf)
{
  if (!strcmp(bf->dispname,"[Template]"))
  {
    loprintf("Already loaded [Template]\n");
    return 1;
  }
  return 0;
}

static bfile_t *load_template(bfile_t *bf)
{
  if (check_if_template(bf)) return bf;
  bfile_save(bf);
  bfile_destroy(bf);
  bf= bfile_open("[Template]");
  gfx_set_text(Wfile, bf->dispname);
  return bf;
}

static void combine_template(bfile_t *bf)
{
  strlist_t *sn;
  bfile_t *tf;
  int o, i; char **B, **T;
  static int str_opts[]=
  {
    OFFSET(c_compiler_path), OFFSET(c_compiler_options),
    OFFSET(c_compiler_optionsR), OFFSET(c_compiler_optionsD),
    OFFSET(c_linker_options), OFFSET(c_linker_optionsR),
    OFFSET(c_linker_optionsD),
    OFFSET(a_build_tool_dir), OFFSET(a_jarfile),
    OFFSET(a_zipalign), OFFSET(a_dx), OFFSET(a_aapt),
    OFFSET(a_javac), OFFSET(a_jarsigner), OFFSET(a_keytool), -1
  };
  tf= bfile_open("[Template]");

  for(i=0;str_opts[i]>=0;i++)
  {
    o= str_opts[i];
    T= (char**)( (char*) tf + o );
    if (isempty(T[0])) continue;
    B= (char**)( (char*) bf + o );
    B[0]= strdup(T[0]);
  }
  
  for(sn=tf->c_include_paths.next;sn!=&tf->c_include_paths;sn=sn->next)
    bfile_add_c_include(bf, sn->name);
  for(sn=tf->c_library_paths.next;sn!=&tf->c_library_paths;sn=sn->next)
    bfile_add_c_library(bf, sn->name);

  bfile_destroy(tf);
}

static bfile_t *new_file(bfile_t *bf)
{
  char *dir,*fn;
  bfile_t *bb;

  bfile_save(bf);
  dir= select_dir(bf->dir, "Select top level directory");
  if (!dir) return bf;
  bb= bfile_open(fn=path_child(dir, "build.abf"));
  loprintf("new file %s\n",fn);
  loprintf("set options and then use Write All\n");
  combine_template(bb);
  bfile_save(bb);
  bfile_destroy(bf);
  return bb;
}

void msg(const char *fmt,...)
{
  char buf[300];
  va_list ap;
  memset(buf,0,sizeof(buf));
  va_start(ap,fmt);
  vsnprintf(buf,sizeof(buf)-1,fmt,ap);
  va_end(ap);
  buf[sizeof(buf)-1]= 0;
  gfx_set_text(Wmessage,buf);
  gfx_timer_start(100, 150);
  while(1)
  {
    gfx_event_t event;
    if (!gfx_event(NULL,&event)) continue;
    if (event.type==GFXEV(100,TIMEOUT)) break;
  } 
}

static void cmd_oo(void *arg, char *data, int len)
{
  gfx_logview_write(Wlog, data,len);
  fwrite(data,1,len,stdout);
  if (cmd_logf)
    (*cmd_logf)(data,len); 
}

char *system_tool(bfile_t *bf, char *field, char *tool)
{
  if (isempty(field)) return tool;
  return field;
}

static int build(bfile_t *bf, int dbg)
{
  if (compile_c_sources(bf, dbg) 
      || make_resources(bf)
      || compile_java_sources(bf) 
      || build_dex(bf) 
      || build_apk(bf))
    return 1;
  loprintf("Build-%c OK\n",dbg);
  return 0;
}

static int upload(bfile_t *bf)
{
  char buf[2000];
  if (check_field(bf->a_adb,"adb")) return 1;
  msg("Uploading APK");
  cmd_reset();
  cmd_args(bf->a_adb, "install", "-r", NULL);
  sprintf(buf,"%s/android/%s.apk", bf->dir, bf->a_appname);
  cmd_args(buf,NULL);
  return cmd_run();
}

static int build_and_upload(bfile_t *bf, int dbg)
{
  if (build(bf,dbg)) return 1;
  if (upload(bf)) return 1;
  loprintf("Upload OK\n");
  return 0;
}

static void clean_files(char *dir, char *ext)
{
  char **files;
  int i;
  files= find_files(dir,ext);
  for(i=0;files[i];i++)
  {
   loprintf("rm %s\n",files[i]);
   unlink(files[i]);
  }
}

static void clean_all(bfile_t *bf)
{
  char *dir;
  int i;
  char *ext[]= { ".class", ".o", ".so", ".dex", ".apk", NULL };
  dir= path_child(bf->dir, "android");
  for(i=0;ext[i];i++)
    clean_files(dir, ext[i]);
  msg("All clean");
}

static void about_screen()
{
  static char *VER= "ABU 1.1  2019-07-26";
  static char *msg[]=
  { "Copyright Şükrü Çınar", NULL };
  int i;
  gfx_event_t event;
  gfx_SCRabout();
  gfx_logview_printf(WaboLOG, "%s\n", VER);
  for(i=0;msg[i];i++) gfx_logview_printf(WaboLOG,"%s\n",msg[i]);
  int q= 0;
  while(!q) if (gfx_event(NULL,&event)) 
   if (event.type==GFXEV_QUIT || event.type==WaboCLOSE) q= 1;
  gfx_leave_screen();
}

static char *result_ip;

#define NOP

static int ispc(int x)
{
  if (x>0 && x<=' ') return 1;
  return 0;
}

static void filter_ip(char *data, int len)
{
  char *e,*k;
  e= data+len;
  for(;data<e && ispc(*data);data++) NOP;
  if (data==e) return ;
  if (strncmp(data, "inet",4)) return ;
  data+= 4;
  if (!ispc(*data)) return ;
  for(;data<e && ispc(*data);data++) NOP;
  if (*data<'0' || *data>'9') return ;
  k= data;
  for(;data<e && (*data=='.' || (*data>='0' && *data<='9')); data++) NOP;
  result_ip= malloc(data-k+1);
  memcpy(result_ip, k, data-k);
  result_ip[data-k]= 0;
}

static char *get_device_ip(bfile_t *bf)
{
  char *result;
  void (*func)(char *,int);
  if (check_field(bf->a_adb,"adb")) return NULL;
  func= cmd_logf;
  result_ip= NULL;
  cmd_logf= filter_ip;
  cmd_reset();
  cmd_args(bf->a_adb, "shell", "ip", "addr", "show", "wlan0", NULL);
  result= NULL;
  if (cmd_run()) { loprintf("adb failed\n"); goto done; }
  result= result_ip;
done:
  result_ip= NULL;
  cmd_logf= func;
  return result;
}

static int go_adb_wifi(bfile_t *bf)
{
  char *ip;
  if (check_field(bf->a_adb,"adb")) return 1;
  msg("Finding out the IP address of device");
  if (!(ip=get_device_ip(bf)))
  {
    loprintf("Failed to query IP address\n");
    return 1;
  }
  loprintf("Device IP is: %s\n", ip);
  msg("Switching ADB over Wifi ON");
  cmd_reset();
  cmd_args(bf->a_adb, "tcpip", "5555", NULL);
  if (cmd_run()) { loprintf("Failed to switch ADB over WiFi ON\n"); return 1; }
  cmd_reset();
  msg("Connecting to device %s", ip);
  cmd_args(bf->a_adb, "connect", ip, NULL);
  if (cmd_run()) { loprintf("Failed to connect <%s>\n", ip); return 1; }
  loprintf("Connected\n");
  return 0;
}

int main(int argc,char **argv)
{
  gfx_event_t event;
  bfile_t *bf;
  int q=0;
  gfx_init(NULL, 800, 600, "Android Builder");
  gfx_SCREEN_0();
  gfx_set_log(Wlog);

  if (argc>1) bf= bfile_open(argv[1]);
         else bf= bfile_open("[Template]");
  gfx_set_text(Wfile, bf->dispname);
  cmd_set_out(cmd_oo, cmd_oo,NULL);
  while(!q) if (gfx_event(NULL,&event)) switch(event.type)
  {
  case GFXEV_QUIT: q= 1; break;
  case Woptions: options_screen(bf); break;
  case Wload: bf= load_file(bf); break;
  case Wnew: bf= new_file(bf); break;
  case Wwrite: write_all(bf); break;
  case WloadTEMPLATE: bf= load_template(bf); break;
  case WcombineTEMPLATE: 
        if (check_if_template(bf)) break;
        combine_template(bf);
        break;
  case WreleaseUPLOAD: build_and_upload(bf,'R'); break;
  case WreleaseBUILD: build(bf,'R'); break;
  case WdebugUPLOAD: build_and_upload(bf,'D'); break;
  case WdebugBUILD: build(bf,'D'); break; 
  case Wclear: gfx_logview_clear(Wlog); msg(""); break;
  case WcleanAll: clean_all(bf); break;
  case Wabout: about_screen(); break;
  case Wadbwifi: go_adb_wifi(bf); break;
  }
  bfile_save(bf);
  gfx_close();
  return 0;
}
