#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include "path.h"

static char *path_pass_kf(char *a, char *b) { if (a!=b) free(b); return a; }

static char *path_strcom2(char *a,char* b)
{
  char *R;
  R= malloc(strlen(a)+strlen(b)+1);
  sprintf(R,"%s%s",a,b);
  return R;
}

static char *path_strcom3(char *a,char* b,char* c)
{
  char *R;
  R= malloc(strlen(a)+strlen(b)+strlen(c)+1);
  sprintf(R,"%s%s%s",a,b,c);
  return R;
}

static char *path_append_slash(char *k) { return path_strcom2(k,"/"); }


char *path_wd()
{
  char *buf;
  size_t siz;

  siz= 1000;
again:
  buf= malloc(siz);
  if (getcwd(buf,siz)==NULL)
  {
     if (errno==ERANGE) { siz+= siz/2; free(buf); goto again; }
     buf[0]= 0;
  }
  return path_pass_kf(path_append_slash(buf),buf);
}   


static int path_has_suffix(char *str,char *sfx)
{
  int Lt, Ls;
  Lt= strlen(str); Ls= strlen(sfx);
  if (Lt<Ls) return 0;
  sfx+= Ls-1;
  str+= Lt-1;
  while(Ls--) if (*(sfx--)!=*(str--)) return 0;
  return 1;
}


static char *path_normalize_name(char *P)
{
  int n;
  char *k,*j;
  P= strdup(P);
  if (path_has_suffix(P,"/..")) P= path_pass_kf(path_strcom2(P,"/"),P);
again:
  n= 1; if ((k=strstr(P,"//"))) goto move;
  n= 2; if ((k=strstr(P,"/./"))) goto move;
  n= 3; k=P; if (!strncmp(P,"/../",4)) goto move;
  if (path_has_suffix(P,"/.")) { P[strlen(P)-1]= 0; goto again; }

  if ((k=strstr(P,"/../")))
  {
    j= k;
    for(k=j-1;k>=P;k--) if (*k=='/') break;
    k++;
    n= j+4-k;
    goto move;
  }
  return P;
move:
  memmove(k,k+n,strlen(k+n)+1);
  goto again;
}

char *path_abs(char *K)
{
  char *R;
  char *wd;
  K= strdup(K);
  if (K[0]!='/') 
  {
    wd= path_wd();
    K= path_pass_kf(path_pass_kf(path_strcom2(wd,K),K),wd);
  }
  return path_pass_kf(path_normalize_name(K),K);
}

char *path_rel(char *fn, char *pivot_dir)
{
  int i,count;
  char *result;
  fn= path_abs(fn); 

  while (fn[0] && fn[strlen(fn)-1]=='/') fn[strlen(fn)-1]= 0;
  if (!fn) return NULL;

  pivot_dir= path_abs(pivot_dir); 

  if (pivot_dir[strlen(pivot_dir)-1]!='/')  
      pivot_dir= path_pass_kf(path_append_slash(pivot_dir),pivot_dir);

  // now, fn is of the form   /f1/f2/f3/f4
  // pivot is of the form     /p1/p2/p3/p4/p5/
  
#ifdef SELF_TEST
  printf("rel: fn=<%s>\n"
         "     pd=<%s>\n", fn, pivot_dir);
#endif

  if (pivot_dir[0]!='/' || fn[0]!='/') return NULL;

again:
  for(i=1;pivot_dir[i]!='/';i++)
    if (fn[i]!=pivot_dir[i]) goto diff;
  if (fn[i]!=pivot_dir[i]) goto diff;
  
  fn+= i;
  pivot_dir+= i;
#ifdef SELF_TEST
  printf("common: fn=<%s>\n"
         "        pd=<%s>\n", fn, pivot_dir);
#endif
  goto again;

diff:
  count= 0;
  for(i=1;pivot_dir[i];i++) if (pivot_dir[i]=='/') count++;
  result= malloc(count*3 + strlen(fn+1) + 1);
  for(i=0;i<count;i++) memcpy(result+i*3, "../",3);
  strcpy(result+count*3,fn+1);
  return result;
}

char *path_dir(char *fn)
{
  char *k;
  fn= strdup(fn);
  while(fn[strlen(fn)-1]=='/') fn[strlen(fn)-1]= 0;
  k= strrchr(fn,'/');
  if (k) { *k= 0; return fn; }
  return path_wd();
}

char *path_file(char *fn)
{
  char *k;
  fn= strdup(fn);
  while(fn[strlen(fn)-1]=='/') fn[strlen(fn)-1]= 0;
  k= strrchr(fn,'/');
  if (k) return k+1; 
  return fn; 
}

char *path_child(char* dir, char *fn)
{
  fn= path_strcom3(dir,"/",fn);
  return path_pass_kf(path_abs(fn),fn);
}

#ifdef SELF_TEST
char *test[]=
{
  "N", "//./foo/bar/../baz/kk.pdf", "",
  "A", ".././../foo/baz/../bar/fifi////./hu.txt", "",
  "R", "/home/dodo/tmp/TTOOL/edi/editor.c", "../TTOOL", 
  "R", "/home/dodo/tmp/TTOOL/edi/editor.c", "../TTOOL/edi", 
  "R", "/home/dodo/tmp/TTOOL/edi/editor.c", "../TTOOL/ebed/build", 
  "R", "/home/dodo/tmp/TTOOL/edi/editor.c", "../TTOOL/ebed/", 
  "C", "../TTOOL/ebed/..", "edi/editor.c", 
  NULL, NULL, NULL,
};

int main(int argc,char **argv)
{
  char *a,*b,*c,*d;
  int i;
  for(i=0;test[i];i+=3)
  {
    a= test[i]; b= test[i+1]; c= test[i+2]; d="";
    switch(a[0])
    {
    case 'N': d= path_normalize_name(b); break;
    case 'A': d= path_abs(b); break;
    case 'R': d= path_rel(b,c); break;
    case 'C': d= path_child(b,c); break;
    }
    printf("%s(%s,%s)=<%s>\n",a,b,c,d);
  }
}
#endif
