#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <sys/stat.h>
#include <errno.h>
#include "bfile.h"
#include "path.h"
#include "main.h"
#include "cmd.h"


static int make_dir(char *fn)
{
  struct stat sb;
  int R;
  msg("mkdir %s", fn);
  R= mkdir(fn,0755);
  if (!R) return 0;
  if (errno!=EEXIST)
  {
    loprintf("mkdir %s: %s\n", fn, strerror(errno));
    return 1;
  }
  if (stat(fn,&sb))
  {
    loprintf("stat %s: %s\n", fn, strerror(errno));
    return 1;
  }
  if (S_ISDIR(sb.st_mode)) return 0;
  loprintf("mkdir %s: file exists but is not a directory\n",fn);
  return 1;
}

static int make_dirs(char *stop,char *name)
{
  char *k;
  if (!strcmp(stop,name)) return 0;
  k= strrchr(name,'/');
  if (!k) return 0;
  *k= 0;
  if (make_dirs(stop,name)) { *k= '/'; return 1; }
  *k= '/';
  return make_dir(name);
}

static FILE *open_file(char *fn, int force)
{
  FILE *f;
  if (!force)
  {
    if (!access(fn,F_OK))
    {
       loprintf("skipping %s: already exists\n",fn);
       return NULL;
    }
  }
  f= fopen(fn,"wb");
  if (!f) loprintf("can't open %s: %s\n",fn, strerror(errno));
  return f;
}

char *package_name_to_dir(char *pak)
{
  char *p;
  int i;
  p= strdup(pak);
  for(i=0;p[i];i++) if (p[i]=='.') p[i]= '/';
  return p;
}

char *java_source_dir(bfile_t *bf)
{
  char *fn;
  fn= path_child(bf->dir,"android/java_sources");
  fn= path_child(fn,package_name_to_dir(bf->a_package));
  return fn;
}

char *java_object_dir(bfile_t *bf)
{
  char *fn;
  fn= path_child(bf->dir,"android/java_objects");
  fn= path_child(fn,package_name_to_dir(bf->a_package));
  return fn;
}

static int make_directories(bfile_t *bf)
{
  int i;
  static char *dirs[]=
  {
    "android",
    "android/resources",
    "android/resources/values",
    "android/resources/drawable-hdpi",
    "android/c_objects",
    "android/lib",
    "android/lib/armeabi",
    "android/bin",
    NULL
  }; 
  for(i=0;dirs[i];i++) if (make_dir(path_child(bf->dir,dirs[i]))) return 1;
  if (isempty(bf->a_package)) 
  {
    loprintf("you must set the package name first\n");
    return 1;
  }
  if (make_dirs(bf->dir, java_source_dir(bf))) return 1;
  if (make_dirs(bf->dir, java_object_dir(bf))) return 1;
  return 0;
}


static int write_strings_xml(bfile_t *bf)
{
  FILE *f;
  if (isempty(bf->a_appname))
  {
    loprintf("set the appname first\n");
    return 1;
  }
  f= open_file(path_child(bf->dir,"android/resources/values/strings.xml"),1);
  if (!f) return 1;
  
  fprintf(f, 
"<?xml version=\0421.0\042 encoding=\042utf-8\042?>\n"
"<resources>\n"
"    <string name=\042app_name\042>%s</string>\n"
"</resources>\n", 
                                   bf->a_appname);
  fclose(f);
  return 0;
}


static int write_appicon(bfile_t *bf)
{
  FILE *f;
  static unsigned char data[]=
  {
0x89,0x50,0x4e,0x47,0x0d,0x0a,0x1a,0x0a,0x00,0x00,0x00,0x0d,
0x49,0x48,0x44,0x52,0x00,0x00,0x00,0x80,0x00,0x00,0x00,0x80,
0x08,0x06,0x00,0x00,0x00,0xc3,0x3e,0x61,0xcb,0x00,0x00,0x00,
0x09,0x70,0x48,0x59,0x73,0x00,0x00,0x0b,0x13,0x00,0x00,0x0b,
0x13,0x01,0x00,0x9a,0x9c,0x18,0x00,0x00,0x11,0x00,0x49,0x44,
0x41,0x54,0x78,0x9c,0xed,0x9d,0x7d,0x78,0x54,0xf5,0x95,0xc7,
0xbf,0xe7,0xce,0x4c,0x12,0xe6,0x25,0x09,0x2f,0xba,0xa0,0xae,
0x5a,0xb0,0x4a,0x0d,0x50,0xc8,0x9d,0x04,0x92,0x49,0x74,0x54,
0x40,0xf0,0xad,0x5d,0xdb,0xd8,0xa7,0x2e,0xab,0x6b,0xb5,0xca,
0xa2,0x6d,0x6d,0x77,0xd5,0xed,0x8b,0x4f,0x1f,0x77,0x6b,0xf5,
0xd1,0x7d,0xc4,0xc7,0xa7,0xdb,0x62,0xa9,0x5b,0x5b,0xec,0xfa,
0x82,0xeb,0xb3,0x22,0xa0,0x8b,0xed,0x1a,0xcc,0x1b,0x21,0x33,
0x01,0x41,0xba,0xac,0x56,0x45,0x57,0xd0,0x02,0x0a,0x99,0x99,
0x9b,0x90,0xcc,0xdc,0xdf,0xd9,0x3f,0xc2,0xc0,0x30,0xb9,0x37,
0x0c,0xe4,0xf7,0x9b,0x17,0xbc,0x9f,0x3f,0x20,0x73,0xcf,0xbd,
0xe7,0x9c,0x99,0x7b,0xe6,0xce,0xbd,0xe7,0x77,0x7e,0xe7,0x07,
0x38,0x38,0x38,0x38,0x38,0x38,0x38,0x38,0x38,0x38,0x38,0x38,
0x38,0x38,0x38,0x38,0x38,0x38,0x38,0x38,0x38,0x38,0x38,0x38,
0x38,0x38,0x38,0x38,0x38,0x38,0x38,0x38,0x38,0x38,0x38,0x38,
0x38,0x9c,0x22,0x50,0xa1,0x1d,0xc8,0x17,0x0c,0x50,0x7f,0x68,
0xce,0x14,0x00,0x67,0xb0,0xa0,0x89,0x4c,0x5a,0x80,0x48,0x94,
0x83,0x88,0x35,0xf0,0x80,0x29,0xd0,0x47,0x4c,0xfb,0x4c,0x94,
0xed,0xae,0xea,0xea,0xfa,0xb4,0xd0,0xfe,0xe6,0x8b,0x53,0x32,
0x00,0x18,0x20,0xa3,0x41,0x9f,0xc1,0x2e,0x34,0x13,0xa3,0x1e,
0xc0,0x6c,0x00,0x17,0x00,0xa8,0xc8,0x51,0xc5,0x01,0x02,0x76,
0x00,0xd8,0xc2,0x84,0x4d,0x9a,0x99,0xda,0xe8,0xed,0x7a,0x63,
0xb7,0x32,0x87,0x0b,0xc8,0x29,0x13,0x00,0xdc,0xd2,0xe2,0x32,
0xf6,0xbc,0x77,0x29,0x13,0xb7,0x10,0xe3,0x2a,0x00,0x53,0xa4,
0x1a,0x20,0x6c,0x67,0x81,0x17,0x01,0x7a,0x36,0xd0,0x19,0x79,
0x53,0xaa,0xee,0x02,0x52,0xf2,0x01,0x90,0x08,0xd7,0x4d,0xe6,
0xa4,0x58,0x46,0xc0,0xcd,0x00,0xce,0xc8,0x8b,0x51,0x46,0x2f,
0x11,0x7e,0xe1,0xad,0xec,0x5b,0x45,0x2f,0xff,0x69,0x30,0x2f,
0x36,0x15,0x51,0xb2,0x01,0x30,0x30,0x6f,0xf6,0xb9,0xc2,0xed,
0xba,0x9b,0x19,0x37,0x21,0xf7,0x4b,0xbb,0x6c,0xf6,0x80,0xf0,
0x88,0xcf,0x9b,0x5c,0x41,0x1b,0xb6,0x19,0x05,0xf2,0x61,0x4c,
0x94,0x5c,0x00,0x7c,0xaa,0xeb,0x55,0x65,0xe5,0xb8,0x17,0x84,
0x6f,0x01,0x28,0x2b,0xb4,0x3f,0x87,0xf9,0x98,0x88,0x7e,0xe8,
0x6d,0x8f,0x3c,0x49,0x80,0x28,0xb4,0x33,0x27,0x42,0x49,0x05,
0x40,0xa2,0x29,0x78,0x03,0x98,0x1f,0x06,0x70,0x7a,0xa1,0x7d,
0xb1,0x84,0xd1,0x4b,0x2e,0x7c,0xd3,0xd7,0x16,0xed,0x2d,0xb4,
0x2b,0xb9,0x52,0x12,0x01,0x10,0x0b,0xeb,0x93,0x5c,0x49,0x7a,
0x9c,0xc1,0xd7,0x16,0xda,0x97,0x1c,0x48,0x12,0xe1,0x9f,0xbc,
0x53,0xa6,0x3e,0x40,0xab,0x57,0x9b,0x85,0x76,0xe6,0x78,0x14,
0x6d,0x00,0x70,0x4b,0x8b,0x2b,0xb1,0x67,0xd7,0x74,0x22,0x4c,
0xd1,0x20,0xe2,0x4c,0x92,0xef,0xea,0x15,0xe3,0x86,0xfb,0xad,
0x94,0x48,0x5d,0xc4,0x8c,0x77,0x45,0xca,0x15,0xad,0xdc,0xbc,
0xf9,0x93,0x42,0xfb,0x64,0x45,0x51,0x05,0xc0,0xc1,0x46,0x7d,
0x9a,0x9b,0x70,0x33,0x01,0x17,0x31,0x30,0x1b,0x84,0x77,0x35,
0x4d,0xbb,0xd2,0xfb,0x7a,0xcf,0xff,0x15,0xda,0xb7,0x93,0x21,
0xde,0x3c,0xe7,0x42,0x12,0xda,0x7a,0x00,0xe7,0x00,0xf8,0x80,
0xc1,0x2f,0x13,0xbb,0x56,0xf8,0x3b,0x7b,0xb6,0x16,0xda,0xb7,
0x34,0x05,0x0f,0x00,0x6e,0x69,0x71,0x0d,0x7c,0xf4,0xce,0x35,
0x82,0x69,0x29,0x80,0x05,0x19,0x3e,0xbd,0x3a,0x98,0x72,0x7f,
0x75,0x62,0x77,0x77,0xac,0x80,0xee,0x8d,0x99,0x44,0xb8,0x6e,
0x32,0x92,0xfc,0x12,0xc0,0xc1,0x8c,0xcd,0xdd,0x60,0x7e,0xdc,
0x57,0x56,0xf9,0x34,0xb5,0xb6,0x1e,0x2a,0x98,0x73,0x28,0x70,
0x00,0xc4,0x9b,0xeb,0x2e,0x26,0x53,0xac,0x04,0xe1,0xf3,0x59,
0xa2,0x27,0x7c,0x9e,0xc0,0x52,0x6a,0x6d,0x4d,0x15,0xc4,0x31,
0xc9,0xb0,0xae,0x7b,0x8d,0x0a,0x7a,0x06,0xe0,0xab,0x8f,0x15,
0xe0,0x6d,0x86,0xb8,0x25,0xd0,0xb9,0xe5,0xf5,0x02,0xb9,0x56,
0x98,0x00,0xf8,0x64,0xee,0xdc,0xca,0x32,0x77,0xf2,0x21,0x02,
0xdd,0x9a,0xed,0x03,0x13,0x56,0x06,0xda,0xa3,0xb7,0x16,0xc2,
0x2f,0x95,0xb0,0xae,0x7b,0x8c,0x0a,0xac,0x05,0xb0,0x30,0x5b,
0xc4,0xc0,0xe3,0x43,0x29,0xf7,0x3d,0x85,0xb8,0xda,0x69,0xf9,
0x36,0xd8,0xdf,0x10,0xac,0x2f,0xf7,0xa4,0x76,0x10,0xe8,0x36,
0x64,0x9d,0x7c,0x02,0xbd,0xe0,0x9f,0x32,0xf5,0xef,0xf2,0xed,
0x53,0x3e,0xa0,0x68,0x34,0xd9,0xef,0x39,0xf4,0x15,0x10,0x36,
0x67,0x8b,0x08,0x58,0x5a,0xee,0x4e,0x6d,0x8f,0xcf,0xab,0xfd,
0x42,0xde,0xfd,0xca,0xa7,0xb1,0x78,0x48,0xbf,0x44,0x03,0xd6,
0x30,0xe0,0xb7,0x10,0xb7,0xfa,0x2a,0xfb,0x16,0x95,0x7a,0x6a,
0xf5,0x78,0xc4,0xc2,0xfa,0x24,0x2d,0x89,0x36,0x00,0xd3,0x2d,
0xc4,0xfb,0x49,0xd0,0x42,0x5f,0x57,0x64,0x4b,0xbe,0xfc,0xc9,
0xdb,0x15,0xc0,0x68,0xaa,0xbd,0x8a,0x80,0xf5,0x36,0x27,0xff,
0x8d,0xc1,0x94,0xfb,0x4b,0xa7,0xfa,0xc9,0x07,0x80,0xca,0xd6,
0xe8,0x7e,0xcd,0xa5,0x2d,0x04,0xf0,0x91,0x85,0x78,0x12,0xbb,
0xf8,0xb5,0x58,0x43,0x5d,0x63,0xbe,0xfc,0xc9,0xcb,0x15,0x20,
0xd1,0xa8,0x2f,0x04,0x61,0x2d,0x00,0x8f,0x85,0x38,0x9e,0x62,
0xcc,0xa9,0xee,0x8c,0xbe,0x93,0x0f,0x5f,0x8a,0x85,0x44,0x53,
0xdd,0x7c,0xb0,0xd8,0x00,0x8b,0x73,0x40,0x40,0x42,0x98,0x5c,
0x1f,0xd8,0xd4,0xfb,0x3f,0xaa,0xfd,0x50,0x7e,0x05,0x48,0x84,
0xeb,0x26,0x83,0xb0,0x0a,0xd6,0x27,0x1f,0x20,0xba,0xfd,0xb3,
0x76,0xf2,0x01,0xc0,0xdf,0xde,0xf3,0x7b,0x10,0x3f,0x6c,0x25,
0x63,0xc0,0x4f,0x2e,0x7a,0x86,0x17,0x9f,0x57,0xae,0xda,0x0f,
0xa5,0x01,0xc0,0x00,0x21,0x69,0xae,0x82,0x6d,0xee,0x9e,0x9e,
0xf2,0xb7,0x47,0x56,0xa9,0xf4,0xa1,0x98,0xf1,0x0d,0xd0,0x8f,
0x00,0xee,0xb1,0x11,0xcf,0xea,0x8f,0x55,0xfe,0x8b,0x6a,0x1f,
0x94,0x06,0x80,0xd1,0xa4,0xdf,0x0d,0xd0,0x7c,0x2b,0x19,0x01,
0xef,0x0c,0xa0,0x7c,0x99,0x4a,0xfb,0xc5,0x0e,0x45,0xa3,0xc9,
0x14,0xd3,0xd7,0x09,0x48,0x58,0xc9,0x19,0x74,0x87,0x11,0x0a,
0x5e,0xa1,0xd2,0x07,0x65,0x01,0x10,0xab,0xaf,0x9f,0x08,0xc6,
0xbd,0x76,0x72,0x66,0x2c,0x3b,0xad,0xa3,0x23,0xae,0xca,0x7e,
0xa9,0x30,0xfc,0xf3,0xc7,0x0f,0xd8,0xc9,0x99,0xf8,0x41,0x56,
0x78,0xaf,0xe6,0x56,0xa5,0x98,0x3c,0xe6,0xb7,0x01,0xf8,0x6c,
0xa4,0xaf,0xf8,0x3b,0x23,0x1b,0x54,0xd9,0x4e,0x13,0x6f,0x0c,
0xce,0x20,0xc2,0x8f,0x00,0x9e,0x41,0xe0,0x9d,0x10,0xda,0xfd,
0xa3,0x3d,0x62,0x31,0x40,0x89,0xc6,0xe0,0xd7,0x08,0x62,0x29,
0x88,0xc6,0x83,0xf1,0x6a,0x4a,0x1b,0xba,0xbf,0xba,0x7d,0xfb,
0x01,0x95,0x7e,0x7a,0x3d,0x95,0x8f,0x18,0xc9,0xf8,0x6d,0x00,
0xce,0xb6,0x70,0x6a,0x66,0x7f,0x28,0x78,0x35,0x3a,0x22,0x6b,
0x54,0xd8,0x56,0x12,0x59,0x7b,0xc3,0x35,0x7e,0x6f,0xb2,0xe2,
0x03,0x00,0xe3,0x2d,0xc4,0x26,0x43,0xfb,0x62,0xa0,0xa3,0x67,
0x87,0x0a,0xdb,0x69,0xe2,0x21,0xfd,0x12,0x02,0xd6,0x01,0x18,
0x97,0xb1,0x79,0x88,0x40,0x7f,0xe5,0xeb,0x88,0xac,0xb7,0x3a,
0x26,0x11,0x0a,0x3e,0x04,0xf0,0x5d,0x59,0x9b,0xdf,0x62,0x4e,
0x36,0x07,0x3a,0xb7,0xed,0x55,0xe6,0x2c,0x80,0x44,0x93,0x7e,
0x3d,0x18,0xbf,0xb3,0x14,0x12,0x36,0xfb,0xdb,0xa3,0x73,0x55,
0xd8,0x55,0xf2,0x13,0xe0,0x1d,0xaa,0xb8,0x15,0xd6,0x27,0x1f,
0x0c,0xfe,0x95,0xea,0x93,0x1f,0xab,0xaf,0x9f,0x48,0xc0,0xb3,
0x38,0xf6,0xe4,0x03,0x40,0x19,0x83,0x7f,0x97,0x08,0xd7,0x4d,
0xce,0x3e,0xc6,0x08,0x05,0xaf,0xb1,0x38,0xf9,0x00,0x70,0x3e,
0x91,0xe7,0xd7,0x4a,0x1c,0xcd,0xc0,0xd7,0x1e,0x7d,0xda,0x22,
0x4b,0x38,0x0c,0xa3,0x3e,0xde,0x5c,0x77,0xb1,0x0a,0xbb,0x4a,
0x02,0x80,0x89,0xbe,0x6a,0x23,0x3a,0x44,0x29,0xcf,0x8f,0x55,
0xd8,0xcc,0x84,0x3c,0xe2,0x7b,0x00,0x4e,0xb3,0x11,0x57,0x23,
0xc9,0xf7,0x64,0x6e,0x60,0x80,0x98,0xf8,0xc1,0x51,0x54,0x5e,
0x11,0x0b,0xd5,0x36,0x4b,0x73,0xd0,0x02,0x02,0x18,0xd0,0x7e,
0x68,0x2b,0x37,0xc5,0xd5,0x76,0xb2,0xb1,0x20,0x3d,0x00,0x3e,
0x99,0x3b,0xb7,0x92,0xc0,0x75,0x36,0xe2,0x67,0xfc,0xdd,0xdd,
0x7f,0x96,0x6d,0x33,0x13,0x06,0x34,0x02,0xdf,0x74,0x9c,0xbd,
0x6e,0x64,0x5d,0x3f,0x92,0x97,0x88,0x37,0xd4,0x35,0x80,0x31,
0x6a,0x1e,0x5e,0x03,0x7d,0x43,0x8e,0x87,0xf6,0x0c,0xe7,0x06,
0x60,0x9d,0xfc,0x21,0x5c,0xaa,0xc2,0xa6,0xf4,0x00,0xa8,0x70,
0x9b,0x61,0xd8,0xdc,0x5c,0x12,0xc4,0xbf,0xca,0xb6,0x97,0x8d,
0xd1,0x5c,0x3b,0x03,0xc7,0x9f,0x13,0x30,0x7e,0x60,0x1c,0x6a,
0xd3,0x2f,0x5c,0x64,0x2e,0xc8,0x41,0xf5,0xe5,0x63,0x72,0x2c,
0x47,0x98,0xe9,0xe7,0x36,0xa2,0xd9,0xb1,0xfa,0xfa,0x89,0xb2,
0xed,0x49,0x0f,0x00,0x26,0xbe,0xcc,0x52,0x40,0xd8,0xec,0xeb,
0xd8,0x12,0x91,0x6d,0x6f,0xa4,0x03,0xda,0x17,0x73,0xda,0x4d,
0xf0,0xac,0xf4,0xdf,0x82,0x28,0x97,0x63,0xa6,0xc4,0xc2,0xfa,
0xa4,0x93,0xf6,0x2b,0x47,0x86,0x52,0xae,0xdf,0xda,0xe4,0x05,
0x88,0xdc,0xe2,0x12,0xd9,0xf6,0xe4,0x07,0xc0,0xb1,0x95,0x2f,
0x19,0x90,0xf2,0x6f,0xff,0xb0,0x03,0xfc,0x17,0x39,0xee,0x99,
0x39,0x89,0x24,0xa7,0x7a,0x43,0x1a,0xa2,0x11,0x37,0x8f,0xb2,
0x99,0xd8,0xdd,0x1d,0x13,0xc0,0x53,0x96,0x42,0x4d,0xcc,0x96,
0x6d,0x4f,0x7a,0x00,0x10,0xd3,0x39,0x16,0x9b,0xfb,0x7d,0x6e,
0xff,0x73,0xb2,0x6d,0xd9,0xd8,0xcf,0x29,0x7f,0x2e,0xb4,0xa3,
0x63,0x13,0x64,0x37,0x4e,0x91,0x85,0xa6,0x09,0xe5,0xb9,0x79,
0x00,0x60,0xa1,0x59,0xa6,0xc7,0x89,0xe9,0x2f,0x65,0xdb,0x52,
0xf1,0x14,0x30,0xf2,0x32,0xc9,0xf8,0xef,0x7c,0xd5,0xbe,0x09,
0x42,0x5f,0x2e,0xfb,0x11,0x28,0x33,0x0b,0x99,0x53,0x46,0x32,
0x65,0xe6,0xa6,0x7b,0xac,0x04,0xce,0x3a,0xb7,0x1b,0xc0,0xc8,
0x19,0xca,0x8c,0x09,0xb2,0x6d,0x49,0x0d,0x00,0x1e,0xd6,0x37,
0xe2,0x5b,0xc2,0x80,0x65,0xe2,0x45,0x05,0x1a,0xf3,0x7b,0xb9,
0xec,0xc7,0x82,0x32,0x47,0x20,0x73,0x19,0x8d,0x4c,0x55,0x96,
0x57,0x7e,0x78,0x72,0x5e,0x9d,0x18,0xb4,0x7a,0xb5,0x09,0xc6,
0x7f,0x8d,0x14,0xb0,0xf4,0x29,0x70,0x52,0x03,0xc0,0x6e,0x5a,
0x94,0x5b,0xc3,0x3a,0x99,0x76,0x46,0xc3,0x2c,0x43,0x37,0x00,
0x3e,0xde,0x7e,0x2e,0xd3,0xb5,0x29,0xfd,0x37,0x01,0x9b,0x46,
0xdb,0xf7,0xf0,0x5e,0x5b,0xf3,0x5b,0xc1,0xcb,0x23,0xbf,0x34,
0x44,0xd2,0x27,0x9a,0x28,0x1b,0x0b,0xc8,0x60,0xc7,0xb8,0xf6,
0xe8,0x07,0x32,0x15,0x32,0x40,0x46,0x48,0xbf,0x01,0x40,0x98,
0x19,0xbd,0xfe,0x41,0xac,0xa0,0x68,0x34,0x09,0x0c,0x57,0xdc,
0x24,0x42,0xfa,0x46,0x00,0xe1,0x51,0x34,0xf4,0x78,0xbb,0xbb,
0x8f,0x7c,0x9b,0xd9,0xa3,0xad,0x45,0x52,0x98,0x00,0x5c,0xb6,
0x87,0x10,0x3f,0x9f,0xf9,0x32,0x1e,0xaa,0xab,0xd1,0x60,0x2e,
0x65,0x68,0xa9,0x14,0xf3,0xcf,0x64,0xd7,0x34,0x88,0x32,0x7a,
0x45,0x4b,0x42,0x40,0xf1,0x88,0xad,0xf2,0x82,0x10,0x62,0x6c,
0x94,0xad,0xb3,0x3f,0x54,0xfb,0x18,0x80,0x27,0x01,0xfc,0x2d,
0x11,0x1e,0x4b,0x54,0xf0,0xf3,0x9c,0xf1,0x5e,0x08,0xb4,0x7c,
0xb4,0xe3,0x99,0xb5,0x47,0x32,0x5f,0xfb,0x5b,0x7b,0x3e,0x86,
0xdd,0x9d,0xf7,0x30,0x71,0x26,0xf1,0x6f,0x19,0xf6,0xe7,0x11,
0x44,0x0f,0x83,0xee,0x00,0xf8,0x4e,0x37,0x21,0x12,0x6f,0x9e,
0x73,0xe1,0x49,0xbd,0x19,0x1b,0x2a,0x5b,0xa3,0xfb,0x01,0xbc,
0x25,0x53,0xa7,0x15,0xca,0x03,0x40,0x00,0x52,0x9b,0x29,0xc4,
0x9b,0x82,0xe1,0xe1,0x0f,0xfe,0x28,0x04,0xba,0xc6,0x68,0x0a,
0x2e,0x49,0xbf,0xf6,0x76,0x44,0x5e,0x62,0xc6,0x7f,0xda,0xa8,
0xd8,0xe0,0xef,0x8c,0x8c,0x7c,0x22,0xf1,0x68,0xff,0x08,0xc0,
0x72,0xc0,0x87,0x40,0x77,0x05,0xda,0xb6,0xec,0x03,0x86,0x27,
0xb2,0x08,0xd0,0xaf,0x71,0xec,0x38,0x43,0x35,0x98,0x56,0xca,
0x1e,0xb6,0x25,0xb0,0xd2,0x31,0x13,0x20,0x2f,0x45,0xa1,0x42,
0xea,0x9b,0xd0,0x58,0xdc,0x61,0x29,0x60,0x7c,0x2b,0xfd,0x27,
0x01,0x3c,0x50,0x76,0xe8,0x6f,0x18,0x9c,0x3d,0x84,0xba,0x61,
0xe8,0x10,0xae,0xb3,0xba,0x57,0xf1,0xb7,0xf6,0x7c,0x0c,0xd6,
0x2e,0x07,0xf0,0x7e,0xc6,0x66,0x13,0xc0,0x0f,0xbc,0x1d,0x91,
0x5f,0xa6,0x37,0x18,0xbb,0xdf,0xbd,0x0c,0x16,0x15,0xbd,0xc4,
0xd4,0xd8,0xdf,0x10,0x94,0xfa,0x9c,0x2e,0x40,0xca,0x03,0x40,
0xf9,0x3d,0x00,0xa7,0x3c,0xd2,0xde,0x04,0x03,0x64,0x80,0xac,
0x33,0x8d,0xe0,0xe0,0x81,0xf0,0xec,0xea,0xf1,0xad,0x5b,0x0f,
0x02,0xc0,0xe9,0xad,0x3b,0x12,0x0c,0x7c,0x79,0xa0,0x49,0xaf,
0x67,0xa6,0x19,0xa6,0x86,0x9d,0x81,0xb6,0x48,0xa7,0x7f,0x94,
0x1b,0x44,0x7f,0x67,0xcf,0x56,0x0e,0x87,0xa7,0xf7,0x0f,0x25,
0x2e,0x67,0xe2,0x6a,0x97,0x69,0x6e,0x1c,0xb7,0x69,0xeb,0xae,
0x63,0x76,0x22,0x9a,0x6f,0xa7,0x82,0x35,0x31,0x1f,0x80,0xc4,
0x92,0x6e,0xde,0xa1,0xba,0x6e,0x57,0x75,0x00,0xec,0x95,0x39,
0x2b,0x36,0x1e,0xd6,0x27,0x6a,0x49,0x54,0xdb,0xc9,0xcb,0x52,
0xda,0x34,0x00,0xd1,0xf4,0x6b,0x02,0x18,0xed,0xd1,0x6e,0x00,
0xdd,0xb9,0xda,0x38,0x7c,0xa7,0xff,0xa2,0xad,0x1c,0x98,0x66,
0x17,0x41,0xcc,0xda,0x79,0xb9,0xda,0xc9,0x0d,0xd7,0x0e,0xd5,
0xfd,0x26,0x14,0x17,0x85,0x92,0xd4,0x3b,0x63,0x4f,0x0a,0xde,
0xd1,0xe4,0x26,0x28,0x7b,0xfc,0x5f,0x3a,0x0c,0x7b,0x1f,0x88,
0x46,0xf7,0xef,0x44,0xf1,0xfb,0x06,0x77,0xc9,0xd4,0x67,0x85,
0xd2,0x2b,0x00,0x41,0x6e,0xe6,0xac,0x3f,0xe9,0x3e,0x58,0xee,
0xb6,0x9f,0x2f,0x4a,0x82,0x0e,0x8e,0x76,0x3c,0x03,0xda,0x40,
0x43,0x30,0x68,0x12,0xeb,0x04,0x4c,0x63,0x42,0x15,0xc0,0x26,
0x08,0xfb,0x49,0x60,0x67,0x0a,0xd4,0x75,0xdc,0xc7,0x39,0xe6,
0x03,0x76,0x57,0x65,0x02,0x4b,0x2d,0x1d,0xa3,0x0d,0xdb,0x8c,
0x44,0x48,0x57,0xfa,0x28,0xa8,0x38,0x00,0x58,0x6a,0xd1,0xe7,
0xc4,0xee,0xee,0x58,0x22,0xa4,0xef,0x02,0x70,0xae,0x85,0x78,
0xc8,0x5f,0x75,0xf0,0x6d,0xab,0xe3,0x06,0x1a,0xe7,0x9c,0x63,
0x12,0x7d,0xc7,0x00,0x2d,0x01,0xf8,0xb4,0xf4,0xf9,0xa3,0xf4,
0xbf,0x3c,0xfc,0x9f,0x1b,0x40,0x22,0xa4,0xff,0x11,0xe0,0x95,
0x3e,0x5f,0x6a,0xa5,0x55,0xe3,0x27,0x22,0x7e,0x93,0x6d,0x23,
0x80,0x55,0xb4,0x8f,0x4b,0x00,0xa8,0x54,0xa0,0x17,0x80,0xea,
0x9f,0x00,0x96,0x1b,0x00,0x00,0x40,0xc0,0x4b,0x96,0xb6,0xc0,
0xaf,0x64,0x4f,0x2d,0x63,0x5d,0xf7,0x26,0x42,0xfa,0xc3,0x26,
0x69,0x6f,0x03,0xf4,0x5d,0xd8,0x57,0x09,0x65,0x72,0x21,0x40,
0xcb,0x0d,0xc3,0xf3,0x6e,0x22,0xa4,0xdf,0x98,0x2d,0x14,0xac,
0xd9,0x15,0x67,0x0a,0x4a,0x7a,0x54,0xa4,0xbc,0x95,0x56,0x4e,
0x2b,0x7e,0x0c,0x24,0xe9,0xce,0x6b,0x2e,0xd7,0x72,0x00,0xd9,
0x29,0x59,0x76,0x01,0xc7,0x94,0x56,0x1b,0xcd,0xfa,0x14,0xa3,
0x02,0x9d,0x00,0xfe,0x01,0x39,0x8e,0xf6,0x65,0x71,0x3a,0x80,
0x27,0x13,0x8d,0xb5,0x4f,0x70,0x38,0x7c,0xe4,0x4a,0x19,0xe8,
0x8c,0xbc,0x69,0xf1,0x78,0x09,0x30,0x3f,0x99,0x99,0x5d,0x94,
0x48,0x09,0x07,0x00,0x61,0x40,0xb6,0xca,0x71,0xaf,0x6f,0x7e,
0x8f,0xc1,0x37,0x02,0x18,0x3a,0xbc,0x89,0x01,0xfe,0x9e,0xb7,
0xa3,0xf7,0x48,0x3e,0x3f,0xd6,0xa0,0x4f,0x67,0x81,0x2e,0x00,
0x39,0x15,0x87,0x8c,0x0a,0xd1,0x37,0x8c,0x64,0xfc,0x45,0x5e,
0x38,0xeb,0x68,0x89,0x3b,0xa7,0xbe,0x89,0x8c,0x04,0x17,0x83,
0xba,0x06,0x4d,0xcf,0x77,0xc7,0x6c,0xcb,0x1a,0xa5,0xe3,0x0f,
0x6a,0x03,0x80,0x47,0x54,0xe5,0x4a,0x21,0xd0,0xd1,0xfb,0x9c,
0x8b,0xf0,0x79,0x66,0xfa,0xba,0x00,0x7d,0xc1,0xdf,0xd1,0xfb,
0x68,0x5a,0x16,0x0b,0x05,0x2f,0xd0,0x34,0x6c,0xc4,0x70,0x5f,
0x1e,0x59,0x5c,0x61,0x18,0x9e,0x97,0xd3,0x41,0x10,0xe8,0xdc,
0xb6,0xd7,0x57,0xd9,0x17,0x24,0xe2,0xab,0x21,0x78,0x91,0xdf,
0xe3,0xbf,0x48,0x55,0x73,0x07,0xb2,0x9e,0x4d,0x2d,0x0d,0xc5,
0x79,0x00,0x0e,0xa8,0xd2,0x7c,0x78,0x80,0xe9,0x98,0x41,0xa6,
0x81,0x26,0xfd,0x6c,0x93,0xf9,0x55,0xa8,0xe9,0x23,0xd8,0x6c,
0x24,0xdc,0xeb,0x58,0xd7,0xaf,0xa0,0x68,0xb4,0xff,0xf0,0xfd,
0xc6,0x5a,0x05,0x76,0x8e,0x81,0x01,0x65,0x9f,0x21,0xa0,0xf8,
0x0a,0x40,0x44,0x4a,0x9d,0xcf,0x24,0xde,0x38,0xeb,0x74,0x53,
0xe0,0xf7,0x00,0xa4,0x57,0xcd,0x1c,0x81,0xe8,0x62,0xa3,0x02,
0x2f,0x71,0x43,0x83,0xf2,0x7c,0x43,0x06,0xa5,0x1b,0x00,0x8c,
0xfc,0x04,0xc0,0x81,0xf0,0xec,0x6a,0x22,0xcf,0x06,0x8b,0x66,
0x53,0x2a,0xb8,0xd4,0xd0,0x06,0xd7,0x70,0x38,0xac,0xbc,0x3f,
0x31,0xb7,0xb4,0xb8,0x30,0x4a,0xe2,0x49,0x06,0x6a,0x03,0x80,
0x84,0xb2,0xe7,0xd7,0x23,0x36,0x16,0xce,0xf2,0xb9,0x93,0xee,
0xf5,0x90,0x71,0xc3,0x97,0x33,0x34,0xdf,0x48,0xc6,0x5f,0x54,
0x3d,0x7f,0xbf,0xef,0xa3,0x9d,0xca,0x3f,0x3f,0xb5,0x3f,0x01,
0x4c,0x53,0x55,0xea,0x67,0x40,0x33,0x0c,0xf7,0x6a,0x02,0x37,
0xa8,0xb4,0x63,0xc3,0xc2,0xfe,0x58,0xf5,0xbf,0xab,0x9c,0xb9,
0x5b,0x86,0xb2,0xf3,0x55,0xe9,0x4e,0xa3,0x7a,0x38,0x78,0x72,
0x5f,0x43,0x83,0xf4,0x42,0xc6,0x34,0x87,0xfb,0x0f,0x2c,0x56,
0xa5,0xff,0x78,0x30,0xf8,0xda,0x44,0x28,0x78,0xbb,0x3a,0xfd,
0x54,0xa3,0x4a,0x77,0x1a,0xf5,0x15,0x41,0xda,0xa0,0x92,0x37,
0xd1,0xd7,0xd0,0x30,0x81,0x18,0xb6,0x73,0xe9,0xf2,0x05,0x81,
0xef,0x3b,0x26,0x47,0x20,0x11,0x06,0x97,0x7e,0x00,0xb8,0x14,
0x45,0xb1,0x46,0x43,0x7f,0x6d,0xd3,0x71,0x2c,0xdf,0x4c,0x30,
0x12,0xee,0x16,0x45,0xba,0x4b,0x3f,0x00,0xa0,0x28,0x8a,0x69,
0x64,0xc7,0xcd,0x02,0x42,0x6a,0x7c,0x61,0xcc,0x50,0xa2,0x37,
0x03,0xe5,0x01,0xc0,0x44,0x4a,0xe6,0xb5,0x13,0x59,0x36,0x5a,
0x2c,0x14,0x17,0xc8,0x56,0x18,0x6b,0xaa,0x3f,0x1f,0xc0,0x99,
0xb2,0xf5,0x66,0xa3,0xfe,0x0a,0xc0,0x98,0xd9,0x3f,0x77,0xee,
0x59,0xf2,0xd5,0x16,0xc5,0xe5,0x7f,0x18,0x92,0xef,0x8b,0xc6,
0xe2,0xca,0x11,0x1b,0x6d,0xc7,0xa1,0xc7,0x60,0x47,0xa6,0x32,
0xb6,0xd1,0xc7,0x9e,0xa4,0x8a,0x4e,0x57,0xa3,0x16,0x7f,0xe4,
0x19,0x05,0x3d,0x84,0x84,0xd5,0x67,0x56,0xdc,0x01,0x70,0xb8,
0xda,0x76,0x28,0x7b,0xbb,0x60,0x92,0x1e,0x00,0x04,0x6c,0x97,
0xad,0x73,0x0c,0x48,0x2d,0x04,0xd9,0x1b,0xae,0xf1,0x03,0x74,
0x51,0xf6,0x76,0x66,0x48,0x5f,0x99,0x4c,0xc5,0x4f,0xc0,0x7e,
0x0b,0x23,0x97,0xc9,0xce,0x9a,0x31,0xf2,0x37,0xdd,0xec,0x78,
0x90,0x26,0x77,0x50,0xc8,0x9f,0x2a,0x5f,0x00,0xab,0x15,0xd1,
0x34,0xec,0x93,0x69,0x67,0x58,0xa5,0x64,0x18,0xf4,0xfe,0xc8,
0x6d,0xf0,0x1b,0xf1,0xaa,0xaf,0xc8,0xb4,0xe3,0x13,0x65,0xcf,
0x01,0xf8,0x58,0xa6,0xce,0x93,0xe4,0x3d,0xaf,0x2b,0x20,0x35,
0x00,0x04,0xd3,0x2d,0x56,0xdb,0x35,0xe6,0x5d,0x32,0xed,0x00,
0x0a,0x02,0x40,0x03,0x2c,0x97,0x4c,0x63,0xb0,0xd4,0x8c,0x19,
0x75,0x75,0x0d,0x80,0xe8,0x6e,0x99,0x3a,0x4f,0xca,0x0f,0xe2,
0xef,0xc8,0x5c,0xd9,0xe4,0x50,0x53,0xdd,0x54,0x00,0x8b,0xac,
0x64,0x02,0xd4,0x25,0xcb,0x4e,0x1a,0xf9,0x0d,0x22,0x48,0xfc,
0xc1,0x72,0x3b,0x53,0x63,0xa2,0xb1,0x4e,0xea,0xcc,0x19,0x7f,
0x7b,0x64,0x15,0x03,0x2b,0x64,0xea,0x3c,0x41,0x1e,0xf0,0xb5,
0xf7,0x5a,0xd6,0x28,0x9e,0x2c,0x29,0xe6,0xa5,0xb0,0x3e,0x2f,
0x87,0xfc,0x95,0x7d,0x9d,0x32,0x6d,0xc1,0xc6,0xd0,0x98,0x18,
0x74,0x8b,0xd7,0x30,0x3c,0xa5,0x6a,0x04,0x4c,0x42,0x7a,0xde,
0xdc,0xdf,0x11,0x5d,0x06,0xe2,0x87,0x90,0xc3,0x94,0x70,0x89,
0x98,0x44,0xb8,0xd7,0xdf,0x11,0xfd,0x81,0x4c,0xa5,0xc3,0x43,
0xcc,0x6c,0xdd,0x8d,0x8c,0xa9,0x43,0xc5,0x7a,0x0a,0xd2,0x03,
0x60,0x7c,0xeb,0xd6,0x83,0xa0,0xa3,0xb3,0x73,0x32,0x21,0xe0,
0x7a,0xd9,0x8d,0x96,0x08,0x60,0x7f,0x7b,0xef,0x3d,0xac,0x69,
0x97,0x30,0xb1,0xf4,0x6f,0xc8,0x08,0x98,0xfe,0xa0,0x09,0x6a,
0xf4,0xb5,0x47,0x7f,0x22,0x5b,0x75,0x7f,0x2a,0x76,0x03,0x00,
0xeb,0x4e,0x60,0xc4,0x96,0x57,0xd6,0xb1,0xa2,0xaa,0x24,0xec,
0x05,0x00,0xf5,0x16,0xdb,0xbd,0xae,0x24,0xff,0x18,0x38,0x3a,
0x91,0x53,0x16,0x81,0xb6,0x9e,0x8d,0x00,0x42,0xb1,0x50,0xf0,
0x02,0x17,0x10,0x66,0x60,0x1a,0x98,0xe5,0xac,0x2d,0x4c,0xe8,
0x07,0xf3,0xdb,0x2e,0x21,0x5e,0x1b,0x31,0x57,0x50,0x12,0x7b,
0xc3,0x35,0x7e,0x4e,0xd2,0x7d,0x36,0x62,0xd3,0x14,0xbc,0x5a,
0x85,0x5d,0x25,0x63,0xd9,0x9f,0xea,0x7a,0x55,0xd9,0x38,0xbc,
0x0f,0x46,0x95,0x85,0x38,0x25,0x40,0x33,0x2a,0x3b,0x22,0xff,
0xab,0xc2,0x76,0xa9,0x12,0x6f,0xd4,0xff,0x79,0xb8,0xb1,0xb5,
0x05,0x8c,0xa7,0xfd,0x9d,0xd1,0xeb,0x55,0xd8,0x55,0x92,0x0a,
0x9e,0x10,0x8d,0xf6,0x81,0x61,0xd7,0xf0,0xd0,0x4d,0x10,0x0f,
0xa9,0xb0,0x5b,0xaa,0xf4,0xcf,0x9d,0x7b,0x16,0x11,0xfe,0xde,
0x46,0xcc,0x0c,0xfa,0xa9,0x2a,0xdb,0xca,0xc6,0x02,0x58,0x13,
0xcb,0x01,0xeb,0x79,0x01,0x04,0xba,0x26,0xde,0x14,0x0c,0xab,
0xb2,0x5d,0x6a,0x08,0x77,0xea,0x7e,0x8c,0x6c,0x6c,0x0d,0x00,
0x60,0x60,0x4d,0xa0,0x33,0xa2,0x62,0xca,0x19,0x00,0x85,0x01,
0x10,0x68,0xdb,0xb2,0x8f,0x08,0xb6,0x91,0x4b,0xcc,0xbf,0x60,
0x5d,0x57,0x5a,0xf0,0x58,0x0a,0x18,0x8d,0xfa,0x95,0x00,0x6e,
0xb0,0x92,0x11,0x90,0x10,0x2e,0xd3,0xaa,0x83,0xb9,0x34,0x94,
0x8e,0x06,0x7a,0xdb,0xa3,0x3f,0x05,0xb3,0x5d,0x8f,0xa0,0xe9,
0x46,0x05,0x1e,0x53,0x69,0xbf,0xd8,0x49,0x84,0xeb,0x26,0x33,
0xc1,0xb6,0x15,0x3d,0x13,0xdd,0x5e,0xf5,0xfa,0x56,0xcb,0x09,
0xaf,0xb2,0x50,0x5b,0x14,0x0a,0x08,0xcd,0xf4,0x2c,0x81,0x55,
0xd3,0xc3,0x61,0x6e,0x8e,0x87,0x6a,0xaf,0x53,0xe9,0x43,0xb1,
0x92,0xb1,0xa0,0x96,0xcd,0x84,0x55,0x7a,0xca,0xdf,0x1e,0xf9,
0xad,0x6a,0x3f,0x94,0xd7,0x03,0x78,0xbb,0xbb,0x3f,0x24,0xd0,
0x4d,0xb0,0x49,0x0e,0x11,0xd1,0x2f,0x07,0xe6,0xcd,0x3e,0x57,
0xb5,0x1f,0xc5,0x86,0xd1,0x18,0xbc,0xcb,0x6e,0x41,0x2d,0x00,
0x6f,0xe6,0x6b,0x41,0xad,0xbc,0xac,0x1c,0xea,0xeb,0x88,0xac,
0x61,0xf0,0xf5,0x00,0x92,0x23,0x84,0x8c,0x2a,0xd3,0xe5,0x5e,
0x3d,0x3c,0x04,0xfa,0xd9,0x20,0xd1,0x54,0xbb,0x08,0xc4,0x36,
0x89,0x24,0xee,0x31,0x45,0xd9,0xc5,0xf9,0x5a,0x50,0x2b,0x6f,
0x4b,0xc7,0x06,0x3a,0x7a,0x9f,0x23,0xd0,0x97,0x61,0xf9,0x64,
0xc0,0x41,0x6f,0xb2,0xe2,0x05,0xae,0xa9,0x91,0x93,0xb8,0x29,
0x62,0xfa,0x43,0xb5,0xf3,0xc0,0xf4,0x1f,0xb0,0x9e,0xb2,0xde,
0x36,0x80,0x71,0x97,0x55,0x75,0x75,0xd9,0xfd,0x64,0x4a,0x27,
0xaf,0xab,0x87,0xfb,0x3a,0x22,0xeb,0x59,0xd3,0x16,0xc3,0xba,
0x1f,0xdf,0x02,0x63,0x7c,0xf9,0x6f,0xec,0xaa,0x8a,0x4e,0x05,
0xe2,0xcd,0x73,0x2e,0x14,0xa0,0x75,0xb0,0x9a,0xee,0x45,0xfc,
0x8c,0xef,0x10,0x16,0xe5,0x7b,0x29,0xbd,0xbc,0xae,0x1e,0x9e,
0x26,0x56,0x5f,0x3f,0x51,0xf3,0x88,0x47,0x01,0x5e,0x92,0x2d,
0x23,0xf0,0xcf,0x7c,0x1d,0xbd,0xd2,0x53,0xc5,0x85,0x66,0xa0,
0x49,0x3f,0xdb,0x04,0x3a,0xc0,0xc8,0xae,0x8f,0xdc,0x43,0x4c,
0xcb,0x7c,0x9d,0x11,0xdb,0xce,0x64,0x2a,0x29,0x48,0x00,0xa4,
0x31,0x42,0xc1,0xc5,0x0c,0x5e,0x81,0xac,0xf5,0xf2,0x88,0xf1,
0x73,0xef,0x99,0x53,0xbf,0x4d,0xab,0x57,0x4b,0x6f,0x8e,0x5c,
0x08,0x12,0x0d,0xfa,0x4c,0x0c,0x37,0xcc,0xce,0x9c,0xb9,0x6c,
0x32,0xd3,0x13,0xc9,0x41,0xbe,0x7b,0x42,0x34,0x9a,0x97,0x36,
0xf4,0x56,0x14,0x34,0x00,0x00,0x80,0x17,0x9f,0x57,0x6e,0xf4,
0x55,0xb6,0x10,0xd1,0x52,0x06,0x42,0x47,0x04,0x84,0x75,0x3e,
0x6f,0xf2,0x6b,0x56,0x8d,0x9a,0x4a,0x89,0x44,0x28,0xb8,0x00,
0xe0,0xe7,0x71,0xb4,0xd1,0xd3,0x6e,0x62,0x5e,0x49,0x6c,0xfe,
0xca,0xdb,0xf5,0xc6,0xee,0x42,0xfa,0x06,0x14,0x41,0x00,0x64,
0x92,0x68,0xd0,0x67,0xc2,0x85,0x5b,0xc0,0xb8,0x08,0x40,0x0d,
0x18,0xdb,0xc9,0x85,0xab,0x7c,0x6d,0xd1,0x8f,0x0a,0xed,0xdb,
0xc9,0x60,0x84,0x82,0x37,0x03,0xfc,0x28,0x83,0x76,0x32,0x44,
0x54,0x63,0xed,0x65,0xef,0x99,0x9f,0x5b,0x5b,0x4c,0x57,0xb6,
0xa2,0x0a,0x80,0x4c,0x78,0xf1,0x79,0xe5,0xfd,0xb1,0xc0,0x4c,
0x26,0x9a,0x04,0x93,0x76,0x0b,0x97,0xe5,0xc8,0x62,0xd1,0x52,
0x96,0x74,0xff,0x39,0xe5,0x19,0xf2,0xfa,0x06,0xb4,0x3f,0xa6,
0x5b,0xd9,0x17,0x23,0x45,0x1b,0x00,0x99,0x0c,0xb7,0x7e,0xc1,
0x6f,0x30,0xea,0x1a,0x00,0x45,0x03,0x03,0xfc,0xa8,0xaf,0x32,
0xf6,0x7d,0x15,0x15,0x3c,0xb2,0x29,0x89,0x00,0x00,0x0e,0xf7,
0x02,0x68,0xd4,0xef,0x24,0xc2,0x7d,0x45,0x35,0x2b,0x28,0x03,
0x02,0xde,0x61,0x4d,0xdc,0xe6,0x6f,0xdb,0xa2,0xa4,0x7a,0x47,
0x05,0x25,0x13,0x00,0x69,0x8c,0xd0,0x9c,0x33,0x98,0xe9,0x41,
0x10,0x2d,0x41,0x91,0xf8,0x4f,0x40,0x82,0x09,0x3f,0xf1,0x1d,
0x38,0xb4,0x9c,0x76,0xec,0x18,0x31,0x31,0xa6,0x98,0x29,0x8a,
0x0f,0xf0,0x64,0x30,0x9a,0xf5,0x5a,0x30,0xbe,0xcf,0x8c,0x6b,
0x51,0xb8,0xe4,0x51,0x0c,0xc4,0x2b,0x90,0xf4,0x3c,0xa2,0x7a,
0x49,0x5c,0x55,0x94,0x6c,0x00,0xa4,0x89,0x35,0xd5,0x9f,0xef,
0x82,0x79,0x27,0x33,0x96,0x40,0x71,0x47,0xad,0x0c,0x76,0x81,
0xf0,0x78,0xd2,0x6d,0xae,0x48,0xaf,0x4f,0x50,0xaa,0x94,0x7c,
0x00,0xa4,0x61,0x5d,0xf7,0x1a,0xe3,0xe8,0x4b,0x2c,0xf8,0x3a,
0x22,0x2c,0x00,0x20,0xbb,0x6b,0xc7,0x1e,0x06,0xd6,0x80,0xe8,
0x59,0x7f,0x7b,0x64,0x23,0xe5,0xb7,0x0c,0x5d,0x19,0xa7,0x4c,
0x00,0x64,0xc2,0x35,0x35,0x65,0x89,0xea,0x8a,0x90,0x06,0x34,
0x33,0x78,0x1e,0x40,0xb3,0x70,0x62,0x73,0xed,0x4d,0x00,0x7f,
0x02,0xf1,0x16,0x16,0xd4,0x45,0x2e,0x6e,0xf5,0xb7,0xf5,0x6e,
0x53,0xe4,0x6e,0x41,0x39,0x25,0x03,0xc0,0x8a,0x7d,0xa1,0x50,
0x60,0x9c,0x36,0xf0,0x39,0x30,0xce,0x80,0xc0,0x24,0x22,0xf2,
0x33,0xa1,0x9c,0x05,0x58,0x03,0x0f,0x08,0x42,0x0c,0xa0,0xbd,
0xc2,0x65,0x7e,0x58,0xf9,0x49,0xf2,0xfd,0x52,0xbb,0x99,0x73,
0x70,0x70,0x70,0x70,0x70,0x70,0x70,0x70,0x70,0x70,0x70,0x70,
0x70,0x70,0x70,0x70,0x70,0x70,0x70,0x70,0x70,0x70,0x70,0x70,
0x70,0x70,0x70,0x70,0x70,0x70,0x70,0x70,0x70,0x70,0x70,0x70,
0x70,0x70,0xf8,0x6c,0xf3,0xff,0xdc,0x69,0xd1,0xf4,0xc3,0x23,
0x26,0x63,0x00,0x00,0x00,0x00,0x49,0x45,0x4e,0x44,0xae,0x42,
0x60,0x82,
  };
  f= open_file(
       path_child(bf->dir,"android/resources/drawable-hdpi/appicon.png"),0);
  if (!f) return 0;
  
  fwrite(data,1,sizeof(data),f);
  fclose(f);
  return 0;
}



static int make_key(bfile_t *bf)
{
  msg("Making key file");
  cmd_reset();
  cmd_args(system_tool(bf, bf->a_keytool, "keytool"),
            "-genkeypair", "-validity", "1000",
            "-dname", "CN=awesome company,O=Android,C=JPN",
            "-keystore", "android/test.keystore",
            "-storepass", "haxorz", "-keypass", "haxorz",
            "-alias", "helljniKey", "-keyalg", "RSA", "-v", NULL);
  return cmd_run();
}

static int write_manifest(bfile_t *bf)
{
  strlist_t *p;
  FILE *f;
  msg("Writing AndroidManifest.xml");
  f= open_file(
       path_child(bf->dir,"android/AndroidManifest.xml"),1);
  if (!f) return 1;
  fprintf(f,
"<?xml version=\0421.0\042 encoding=\042utf-8\042?>\n"
"<!-- BEGIN_INCLUDE(manifest) -->\n"
"<manifest xmlns:android=\042http://schemas.android.com/apk/res/android\042\n"
"          package=\042%s\042\n"
"          android:versionCode=\0421\042\n"
"          android:versionName=\0421.0\042>\n"
"\n"
"    <!-- This is the platform API where NativeActivity was introduced. -->\n"
"    <uses-sdk android:minSdkVersion=\0429\042 />\n",
   bf->a_package);

  if (bf->scalign=='P')
   fprintf(f,
"    <uses-feature android:name=\042android.hardware.screen.portrait\042 />\n"
   ); else fprintf(f,
"    <uses-feature android:name=\042android.hardware.screen.landscape\042 />\n"
   );
   fprintf(f, "\n");

   for(p=bf->perms.next;p!=&bf->perms;p=p->next)
    fprintf(f,
"    <uses-permission android:name=\042android.permission.%s\042 />\n",
    p->name);
   fprintf(f, "\n"
"    <application android:label=\042@string/app_name\042 \n"
"                 android:hasCode=\042true\042\n"
"                 android:debuggable=\042true\042\n"
"                 android:icon=\042@drawable/appicon\042 >\n"
"\n"
"        <activity android:name=\042.MyActivity\042\n"
"                  android:label=\042@string/app_name\042\n"
"                  android:screenOrientation=\042nosensor\042\n"
"                  android:configChanges=\042keyboardHidden\042>\n"
"\n"
"            <!-- Tell NativeActivity the name of or .so -->\n"
"            <meta-data android:name=\042android.app.lib_name\042\n"
"                       android:value=\042main\042 />\n"
"            <intent-filter>\n"
"                <action android:name=\042android.intent.action.MAIN\042 />\n"
"                <category android:name=\042android.intent.category.LAUNCHER\042 />\n"
"            </intent-filter>\n"
"        </activity>\n"
"    </application>\n"
"\n"
"</manifest> \n"
"<!-- END_INCLUDE(manifest) -->\n");
   fclose(f);
   return 0;
} 

static int write_myactivity_java(bfile_t *bf)
{
  FILE *f;
  msg("Writing MyActivity.java");
  f= open_file(path_child(java_source_dir(bf),"MyActivity.java"),1);
  if (!f) return 1;
     fprintf(f,
"\n"
"package %s;\n", 
     bf->a_package);
     fprintf(f,
"\n"
"import android.app.Activity;\n"
"import android.content.pm.ActivityInfo;\n"
"import android.app.NativeActivity;\n"
"import android.os.Environment;\n"
"import android.content.Intent;\n"
"import android.content.ClipData;\n"
"import android.database.Cursor;\n"
"import android.graphics.BitmapFactory;\n"
"import android.net.Uri;\n"
"import android.os.Bundle;\n"
"import android.provider.MediaStore;\n"
"import android.util.DisplayMetrics;\n"
"import android.content.ActivityNotFoundException;\n"
"import java.util.ArrayList;\n"
" \n"
"public class MyActivity extends NativeActivity \n"
"{\n"
" \n"
"static { System.loadLibrary(\042main\042); }\n"
"\n"
"public static native void haveresult(int rcode);\n"
"\n"
"private String gdd_s1;\n"
"private String cif_s1;\n"
"\n"
"private static final int RESULT_CHOOSE_IMAGE = 1;\n"
"private static final int RESULT_CHOOSE_IMAGES = 2;\n"
"\n"
"public void get_download_dir()\n"
"{\n"
"   gdd_s1= Environment.getExternalStoragePublicDirectory\n"
"                (Environment.DIRECTORY_DOWNLOADS).getAbsolutePath();\n"
"}\n"
"\n"
"public void choose_image_file()\n"
"{\n"
"    cif_s1= null;\n"
"    Intent i= new Intent(Intent.ACTION_PICK,\n"
"              android.provider.MediaStore.Images.Media.EXTERNAL_CONTENT_URI);\n"
"    startActivityForResult(i, RESULT_CHOOSE_IMAGE);\n"
"}\n"
"\n"
"void choose_image_file_result(int requestCode,int resultCode, Intent data)\n"
"{\n"
"    if (resultCode!=RESULT_OK || data==null)\n"
"    {\n"
"      cif_s1= null;\n"
"      haveresult(requestCode);\n"
"      return ;\n"
"    }\n"
"    Uri selectedImage = data.getData();\n"
"    String[] filePathColumn = { MediaStore.Images.Media.DATA };\n"
"\n"
"    Cursor cursor = getContentResolver().query(selectedImage,\n"
"                       filePathColumn, null, null, null);\n"
"    cursor.moveToFirst();\n"
"\n"
"    int columnIndex = cursor.getColumnIndex(filePathColumn[0]);\n"
"    String picturePath = cursor.getString(columnIndex);\n"
"    cursor.close();\n"
"    cif_s1= picturePath;\n"
"    haveresult(requestCode);\n"
"}\n"
"\n"
"public void choose_image_files()\n"
"{\n"
"   cif_s1= null;\n"
"   Intent intent = new Intent();\n"
"   intent.setType(\042image/*\042);\n"
"   intent.putExtra(Intent.EXTRA_ALLOW_MULTIPLE, true);\n"
"   intent.setAction(Intent.ACTION_GET_CONTENT);\n"
"   startActivityForResult(Intent.createChooser(intent,\042Select Picture\042), \n"
"                  RESULT_CHOOSE_IMAGES);\n"
"}\n"
"\n"
"String file_path_of_uri(Uri uri)\n"
"{\n"
"  String value;\n"
"  String[] filePathColumn = { MediaStore.Images.Media.DATA };\n"
"  if (uri==null)\n"
"  {\n"
"    return \042BadUri\042;\n"
"  }\n"
"  Cursor cursor = getContentResolver().query(uri, \n"
"                             filePathColumn, null, null, null);\n"
"  cursor.moveToFirst();\n"
"  int columnIndex = cursor.getColumnIndex(filePathColumn[0]);\n"
"  value= cursor.getString(columnIndex);\n"
"  if (value==null) value= \042BadStr\042;\n"
"  cursor.close();\n"
"  return value;\n"
"}\n"
"\n"
"void choose_image_files_result(int requestCode,int resultCode, Intent data)\n"
"{\n"
"  ArrayList<String> imagelist;\n"
"  int i;\n"
"  if (resultCode!=RESULT_OK || data==null)\n"
"  {\n"
"    cif_s1= null;\n"
"    haveresult(requestCode);\n"
"    return ;\n"
"  }\n"
"\n"
"  // Get the Image from data\n"
"  String[] filePathColumn = { MediaStore.Images.Media.DATA };\n"
"  imagelist = new ArrayList<String>();\n"
"\n"
"  if (data.getClipData() != null) \n"
"  {\n"
"     ClipData mClipData = data.getClipData();\n"
"     String s, p;\n"
"     Uri u;\n"
"     for (i=0;i<mClipData.getItemCount();i++) \n"
"     {\n"
"       u= mClipData.getItemAt(i).getUri();\n"
"       if (!u.getScheme().equals(\042file\042)) s= u.toString();\n"
"       else\n"
"       s= u.getPath();  \n"
"       if (s!=null) imagelist.add(s);\n"
"     }\n"
"  } \n"
"  else if (data.getData()!=null)\n"
"  {\n"
"    imagelist.add(file_path_of_uri(data.getData()));\n"
"  }\n"
"\n"
"  StringBuffer sb;\n"
"  sb= new StringBuffer();\n"
"  for(i=0;i<imagelist.size();i++)\n"
"  {\n"
"    if (i!=0) sb.append(\042\134n\042);\n"
"    sb.append(imagelist.get(i));\n"
"  }\n"
"\n"
"  cif_s1= sb.toString();\n"
"  haveresult(requestCode);\n"
"}\n"
"  \n"
"@Override\n"
"protected void onActivityResult\n"
"      (int requestCode, int resultCode, Intent data) \n"
"{\n"
"  super.onActivityResult(requestCode, resultCode, data);\n"
"\n"
"  switch(requestCode)\n"
"  {\n"
"  case RESULT_CHOOSE_IMAGE:\n"
"    choose_image_file_result(requestCode,resultCode, data);\n"
"    break;\n"
"  case RESULT_CHOOSE_IMAGES:\n"
"    choose_image_files_result(requestCode,resultCode, data);\n"
"    break;\n"
"  }\n"
"}\n"
"\n"
"@Override\n"
"protected void onCreate(Bundle savedInstanceState) {\n"
"   super.onCreate(savedInstanceState);\n"
"   // setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);\n"
"   setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_SENSOR_PORTRAIT);\n"
"}\n"
"\n"
"public int getStatusBarHeight() \n"
"{\n"
"  int result= 0;\n"
"  int resourceId= getResources().getIdentifier\n"
"                          (\042status_bar_height\042, \042dimen\042, \042android\042);\n"
"  if (resourceId > 0)\n"
"  {\n"
"      result= getResources().getDimensionPixelSize(resourceId);\n"
"  }\n"
"  return result;\n"
"}\n"
"\n"
"public int getDisplayDPI()\n"
"{\n"
"  DisplayMetrics dm = new DisplayMetrics();\n"
"  getWindowManager().getDefaultDisplay().getMetrics(dm);\n"
"\n"
"\n"
"  float xDpi = dm.xdpi;\n"
"  float yDpi = dm.ydpi;\n"
"\n"
"  if (xDpi>yDpi) return (int) xDpi; else return (int) yDpi;\n"
"}\n"
"\n"
"public void rateApp()\n"
"{\n"
"  Uri uri = Uri.parse(\042market://details?id=\042 + getPackageName());\n"
"  Intent goToMarket = new Intent(Intent.ACTION_VIEW, uri);\n"
"  // To count with Play market backstack, After pressing back button, \n"
"  // to taken back to our application, we need to add following flags to intent. \n"
"  goToMarket.addFlags(Intent.FLAG_ACTIVITY_NO_HISTORY |\n"
"                    // Intent.FLAG_ACTIVITY_NEW_DOCUMENT |\n"
"                    Intent.FLAG_ACTIVITY_MULTIPLE_TASK);\n"
"  try\n"
"  {\n"
"    startActivity(goToMarket);\n"
"  }\n"
"  catch (ActivityNotFoundException e)\n"
"  {\n"
"      startActivity(new Intent(Intent.ACTION_VIEW,\n"
"          Uri.parse(\042http://play.google.com/store/apps/details?id=\042 \n"
"                      + getPackageName())));\n"
"  }\n"
"}\n"
"\n"
"\n"
"}\n"
"\n"
      );
  fclose(f);
  return 0; 
}

void write_all(bfile_t *bf)
{
  if (chdir(bf->dir)) 
  {
    loprintf("chdir(%s): %s\n", bf->dir, strerror(errno));
    return ;
  }
  if (make_directories(bf)) return ;
  if (write_strings_xml(bf)) return ;
  if (write_appicon(bf)) return ;
  if (write_manifest(bf)) return ;
  if (write_myactivity_java(bf)) return ;
  if (make_key(bf)) return ;
  msg("All done");
}
